package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.config.OSSConfig;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.OSSUtil;
import com.jpsoft.employment.modules.common.utils.POIUtils;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.EnterpriseInfo;
import com.jpsoft.employment.modules.base.service.EnterpriseInfoService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.ResultMap;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/enterpriseInfo")
public class EnterpriseInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;

    @Autowired
    private OSSConfig ossConfig;


    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<EnterpriseInfo> create() {
        MessageResult<EnterpriseInfo> msgResult = new MessageResult<>();

        EnterpriseInfo enterpriseInfo = new EnterpriseInfo();

        msgResult.setData(enterpriseInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    public MessageResult<EnterpriseInfo> add(@RequestBody EnterpriseInfo enterpriseInfo, @RequestAttribute String subject) {
        MessageResult<EnterpriseInfo> msgResult = new MessageResult<>();

        try {
            enterpriseInfo.setId(UUID.randomUUID().toString());
            enterpriseInfo.setDelFlag(false);
            enterpriseInfo.setCreateBy(subject);
            enterpriseInfo.setCreateTime(new Date());
            enterpriseInfo.setStatus("0");

            int affectCount = enterpriseInfoService.insert(enterpriseInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(enterpriseInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<EnterpriseInfo> edit(@PathVariable("id") String id) {
        MessageResult<EnterpriseInfo> msgResult = new MessageResult<>();

        try {
            EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(id);

            if (enterpriseInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(enterpriseInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    public MessageResult<EnterpriseInfo> update(@RequestBody EnterpriseInfo enterpriseInfo, @RequestAttribute String subject) {
        MessageResult<EnterpriseInfo> msgResult = new MessageResult<>();

        try {
            enterpriseInfo.setUpdateBy(subject);
            enterpriseInfo.setUpdateTime(new Date());

            int affectCount = enterpriseInfoService.update(enterpriseInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(enterpriseInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<EnterpriseInfo> delete(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<EnterpriseInfo> msgResult = new MessageResult<>();

        try {

            EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(id);
            enterpriseInfo.setDelFlag(true);
            enterpriseInfo.setUpdateBy(subject);
            enterpriseInfo.setUpdateTime(new Date());

            int affectCount = enterpriseInfoService.update(enterpriseInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(id);
                enterpriseInfo.setDelFlag(true);
                enterpriseInfo.setUpdateBy(subject);
                enterpriseInfo.setUpdateTime(new Date());

                affectCount += enterpriseInfoService.update(enterpriseInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Object> pageList(
            String name,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            @RequestParam(value = "exportFlag", defaultValue = "false") Boolean exportFlag,
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Object> msgResult = new MessageResult<>();

        try {

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (StringUtils.isNotEmpty(name)) {
                searchParams.put("name", "%" + name + "%");
            }


            Page<EnterpriseInfo> page = enterpriseInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<Map> mapList = new ArrayList<>();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            for (EnterpriseInfo enterpriseInfo : page) {
                Map map = new HashMap();
                map.put("id", enterpriseInfo.getId());
                map.put("name", enterpriseInfo.getName());
                map.put("address", enterpriseInfo.getAddress());
                map.put("licenseImage", enterpriseInfo.getLicenseImage());
                map.put("contactsPersonId", enterpriseInfo.getContactsPersonId());

                String statusN;
                if ("1".equals(enterpriseInfo.getStatus())) {
                    statusN = "已认证";
                } else {
                    statusN = "未认证";
                }

                map.put("status", enterpriseInfo.getStatus());
                map.put("statusN", statusN);

                String checkTime = "";
                if (enterpriseInfo.getCheckTime() != null) {
                    checkTime = sdf.format(enterpriseInfo.getCheckTime());
                }

                map.put("checkTime", checkTime);

                map.put("contactsPhone", enterpriseInfo.getContactsPhone());

                mapList.add(map);
            }

            if (exportFlag) {
                String filePath = exportXls(mapList);
                msgResult.setData(filePath);
            } else {
                Map<String, Object> dataMap = PojoUtils.pageWrapper(page);
                dataMap.put("data", mapList);
                msgResult.setData(dataMap);
            }

            msgResult.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;
    }


    private String exportXls(List<Map> mapList) {
        String downloadUrl = "";

        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        //表头
        Row rowTitle = sheet.createRow(0);

        String[] titles = new String[]{"序号", "企业名称", "企业地址", "企业联系人","联系电话", "认证状态", "认证时间"};

        for (int i = 0; i < titles.length; i++) {
            Cell cell = rowTitle.createCell(i);
            cell.setCellValue(titles[i]);
        }

        for (int i = 0; i < mapList.size(); i++) {
            Map<String, Object> map = mapList.get(i);

            Row row = sheet.createRow(i + 1);

            int colIndex = 0;
            row.createCell(colIndex++).setCellValue(i + 1);
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("name"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("address"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("contactsPersonId"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("contactsPhone"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("statusN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("checkTime"), ""));
        }

        ByteArrayOutputStream output = new ByteArrayOutputStream();

        try {
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            downloadUrl = OSSUtil.upload(ossConfig, "EnterpriseList", "企业列表.xls", input);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }

        return downloadUrl;
    }


    @ApiOperation(value = "列表")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    public MessageResult<List<EnterpriseInfo>> list(
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<List<EnterpriseInfo>> msgResult = new MessageResult<>();

        List<EnterpriseInfo> list = enterpriseInfoService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }


    @ApiOperation(value = "审核信息")
    @PostMapping("check/{id}")
    public MessageResult<EnterpriseInfo> check(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<EnterpriseInfo> msgResult = new MessageResult<>();

        try {

            Date now = new Date();
            EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(id);
            enterpriseInfo.setStatus("1");
            enterpriseInfo.setUpdateBy(subject);
            enterpriseInfo.setUpdateTime(now);
            enterpriseInfo.setCheckTime(now);

            int affectCount = enterpriseInfoService.update(enterpriseInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库认证失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "导入企业")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadFile", value = "上传文件", required = true, paramType = "form", dataType = "__file")
    })
    public MessageResult<String> importXls(MultipartFile uploadFile, @RequestAttribute String subject) {
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 3;

            int count = sheet1.getLastRowNum();

            for (int rowIndex = 1; rowIndex <= count; rowIndex++) {
                try {

                    String name = poiUtils.getCellValue(sheetIndex, rowIndex, 0).toString().replace(" ", "");
                    String address = poiUtils.getCellValue(sheetIndex, rowIndex, 1).toString().replace(" ", "");

                    if (StringUtils.isEmpty(name)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写企业名称!");
                        failCount++;
                        continue;
                    }

                    EnterpriseInfo currItem = enterpriseInfoService.findByName(name);
                    if (currItem != null) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("该企业名称已存在系统当中，不能重复添加!");
                        failCount++;
                        continue;
                    }

                    //企业
                    EnterpriseInfo enterpriseInfo = new EnterpriseInfo();
                    enterpriseInfo.setId(UUID.randomUUID().toString());
                    enterpriseInfo.setName(name);
                    enterpriseInfo.setAddress(address);
                    enterpriseInfo.setStatus("1");
                    enterpriseInfo.setCreateBy(subject);
                    enterpriseInfo.setCreateTime(new Date());
                    enterpriseInfo.setDelFlag(false);

                    //保存新企业
                    enterpriseInfoService.insert(enterpriseInfo);
                    affectCount++;
                } catch (Exception innerEx) {
                    logger.error(innerEx.getMessage(), innerEx);
                }
            }

            if (failCount > 0) {
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex, validateColIndex, 1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig, "import", "error.xls", input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            } else {
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "判断企业名称是否重复")
    @PostMapping("nameIsRepeatCheck")
    public MessageResult<Boolean> nameIsRepeatCheck(String id, String name) {
        MessageResult<Boolean> msgResult = new MessageResult<>();

        try {
            Boolean flag = false;

            EnterpriseInfo enterpriseInfo = enterpriseInfoService.findByName(name);

            if (enterpriseInfo == null) {
                flag = true;
            } else {
                if (id.equals(enterpriseInfo.getId())) {
                    flag = true;
                }
            }

            msgResult.setResult(true);
            msgResult.setData(flag);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
