package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.base.entity.EnterpriseInfo;
import com.jpsoft.employment.modules.base.entity.RecruitInformationInfo;
import com.jpsoft.employment.modules.base.service.EnterpriseInfoService;
import com.jpsoft.employment.modules.base.service.RecruitInformationInfoService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.JobEnterpriseRelation;
import com.jpsoft.employment.modules.base.service.JobEnterpriseRelationService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.ResultMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/jobEnterpriseRelation")
public class JobEnterpriseRelationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private JobEnterpriseRelationService jobEnterpriseRelationService;

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;

    @Autowired
    private RecruitInformationInfoService recruitInformationInfoService;
	
	
	@ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<JobEnterpriseRelation> create(){
        MessageResult<JobEnterpriseRelation> msgResult = new MessageResult<>();

        JobEnterpriseRelation jobEnterpriseRelation = new JobEnterpriseRelation();

        msgResult.setData(jobEnterpriseRelation);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<JobEnterpriseRelation> add(@RequestBody JobEnterpriseRelation jobEnterpriseRelation,@RequestAttribute String subject){
        MessageResult<JobEnterpriseRelation> msgResult = new MessageResult<>();

        try {
            jobEnterpriseRelation.setId(UUID.randomUUID().toString());
			jobEnterpriseRelation.setDelFlag(false);
            jobEnterpriseRelation.setCreateBy(subject);
            jobEnterpriseRelation.setCreateTime(new Date());

            int affectCount = jobEnterpriseRelationService.insert(jobEnterpriseRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(jobEnterpriseRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<JobEnterpriseRelation> edit(@PathVariable("id") String id){
        MessageResult<JobEnterpriseRelation> msgResult = new MessageResult<>();

        try {
            JobEnterpriseRelation jobEnterpriseRelation = jobEnterpriseRelationService.get(id);

            if (jobEnterpriseRelation != null) {
                msgResult.setResult(true);
                msgResult.setData(jobEnterpriseRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<JobEnterpriseRelation> update(@RequestBody JobEnterpriseRelation jobEnterpriseRelation,@RequestAttribute String subject){
        MessageResult<JobEnterpriseRelation> msgResult = new MessageResult<>();

        try {
		    jobEnterpriseRelation.setUpdateBy(subject);
            jobEnterpriseRelation.setUpdateTime(new Date());
		
            int affectCount = jobEnterpriseRelationService.update(jobEnterpriseRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(jobEnterpriseRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<JobEnterpriseRelation> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<JobEnterpriseRelation> msgResult = new MessageResult<>();

        try {
			
			JobEnterpriseRelation jobEnterpriseRelation = jobEnterpriseRelationService.get(id);
            jobEnterpriseRelation.setDelFlag(true);
            jobEnterpriseRelation.setUpdateBy(subject);
            jobEnterpriseRelation.setUpdateTime(new Date());
			
			int affectCount = jobEnterpriseRelationService.update(jobEnterpriseRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                JobEnterpriseRelation jobEnterpriseRelation = jobEnterpriseRelationService.get(id);
                jobEnterpriseRelation.setDelFlag(true);
                jobEnterpriseRelation.setUpdateBy(subject);
                jobEnterpriseRelation.setUpdateTime(new Date());

                affectCount += jobEnterpriseRelationService.update(jobEnterpriseRelation);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String informationId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(informationId)) {
            searchParams.put("informationId", informationId);
        }

        int updateCount = 0;


        Page<JobEnterpriseRelation> page = jobEnterpriseRelationService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (JobEnterpriseRelation jobEnterpriseRelation:page) {
            EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(jobEnterpriseRelation.getEnterpriseId());
            if(enterpriseInfo!=null){
                jobEnterpriseRelation.setEnterpriseName(enterpriseInfo.getName());
                jobEnterpriseRelation.setEnterpriseAddress(enterpriseInfo.getAddress());
            }


            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(jobEnterpriseRelation.getRecruitInformationId());

            if(recruitInformationInfo!=null){
                jobEnterpriseRelation.setRecruitPositionName(recruitInformationInfo.getPositionName());
            }

            if(jobEnterpriseRelation.getIsRead()==null||!jobEnterpriseRelation.getIsRead()){
                jobEnterpriseRelation.setIsRead(true);
                jobEnterpriseRelation.setUpdateBy(subject);
                jobEnterpriseRelation.setUpdateTime(new Date());

                updateCount += jobEnterpriseRelationService.update(jobEnterpriseRelation);
            }
        }

        Map map = PojoUtils.pageWrapper(page);

        map.put("updateCount",updateCount);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }
}
