package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.config.OSSConfig;
import com.jpsoft.employment.modules.base.entity.JobEnterpriseRelation;
import com.jpsoft.employment.modules.base.entity.PersonInfo;
import com.jpsoft.employment.modules.base.service.JobEnterpriseRelationService;
import com.jpsoft.employment.modules.base.service.PersonInfoService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.OSSUtil;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.JobInformationInfo;
import com.jpsoft.employment.modules.base.service.JobInformationInfoService;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/jobInformationInfo")
public class JobInformationInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private JobInformationInfoService jobInformationInfoService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private JobEnterpriseRelationService jobEnterpriseRelationService;

    @Autowired
    private OSSConfig ossConfig;


    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<JobInformationInfo> create() {
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        JobInformationInfo jobInformationInfo = new JobInformationInfo();

        msgResult.setData(jobInformationInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    public MessageResult<JobInformationInfo> add(@RequestBody JobInformationInfo jobInformationInfo, @RequestAttribute String subject) {
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        try {
            jobInformationInfo.setId(UUID.randomUUID().toString());
            jobInformationInfo.setDelFlag(false);
            jobInformationInfo.setCreateBy(subject);
            jobInformationInfo.setCreateTime(new Date());
            jobInformationInfo.setUpdateTime(new Date());

            int affectCount = jobInformationInfoService.insert(jobInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(jobInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<JobInformationInfo> edit(@PathVariable("id") String id) {
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        try {
            JobInformationInfo jobInformationInfo = jobInformationInfoService.get(id);

            if (jobInformationInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(jobInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    public MessageResult<JobInformationInfo> update(@RequestBody JobInformationInfo jobInformationInfo, @RequestAttribute String subject) {
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        try {
            jobInformationInfo.setUpdateBy(subject);
            jobInformationInfo.setUpdateTime(new Date());

            int affectCount = jobInformationInfoService.update(jobInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(jobInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<JobInformationInfo> delete(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        try {

            JobInformationInfo jobInformationInfo = jobInformationInfoService.get(id);
            jobInformationInfo.setDelFlag(true);
            jobInformationInfo.setUpdateBy(subject);
            jobInformationInfo.setUpdateTime(new Date());

            int affectCount = jobInformationInfoService.update(jobInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                JobInformationInfo jobInformationInfo = jobInformationInfoService.get(id);
                jobInformationInfo.setDelFlag(true);
                jobInformationInfo.setUpdateBy(subject);
                jobInformationInfo.setUpdateTime(new Date());

                affectCount += jobInformationInfoService.update(jobInformationInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Object> pageList(
            String name, String intendedIndustries, String intendedPosition, String method,
            String workExperience, String education, String status,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            @RequestParam(value = "exportFlag", defaultValue = "false") Boolean exportFlag,
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Object> msgResult = new MessageResult<>();

        try {

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time", "desc"));

            if (StringUtils.isNotEmpty(name)) {
                searchParams.put("name", "%" + name + "%");
            }

            if (StringUtils.isNotEmpty(intendedPosition)) {
                searchParams.put("intendedPosition", "%" + intendedPosition + "%");
            }

            if (StringUtils.isNotEmpty(intendedIndustries)) {
                searchParams.put("intendedIndustries", intendedIndustries);
            }

            if (StringUtils.isNotEmpty(method)) {
                searchParams.put("method", method);
            }

            if (StringUtils.isNotEmpty(workExperience)) {
                searchParams.put("workExperience", workExperience);
            }

            if (StringUtils.isNotEmpty(education)) {
                searchParams.put("education", education);
            }

            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }


            Page<JobInformationInfo> page = jobInformationInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<Map> mapList = new ArrayList<>();

            for (JobInformationInfo jobInformationInfo : page) {
                Map map = new HashMap();
                map.put("id", jobInformationInfo.getId());

                String personName = "";
                PersonInfo personInfo = personInfoService.get(jobInformationInfo.getPersonId());
                if (personInfo != null) {
                    personName = personInfo.getRealName();
                }

                String intendedIndustriesN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业", jobInformationInfo.getIntendedIndustries());
                if (StringUtils.isNotEmpty(intendedIndustriesN)) {
                    jobInformationInfo.setIntendedIndustriesN(intendedIndustriesN);
                }

                String methodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式", jobInformationInfo.getMethod());
                if (StringUtils.isNotEmpty(methodN)) {
                    jobInformationInfo.setMethodN(methodN);
                }

                String workExperienceN = dataDictionaryService.findNameByCatalogNameAndValue("工作经验", jobInformationInfo.getWorkExperience());
                if (StringUtils.isNotEmpty(workExperienceN)) {
                    jobInformationInfo.setWorkExperienceN(workExperienceN);
                }


                String educationN = dataDictionaryService.findNameByCatalogNameAndValue("学历", jobInformationInfo.getEducation());
                if (StringUtils.isNotEmpty(educationN)) {
                    jobInformationInfo.setEducationN(educationN);
                }

                Integer isReadNumber = jobEnterpriseRelationService.countByJobInformationId(jobInformationInfo.getId());

                jobInformationInfo.setIsReadNumber(isReadNumber);


                map.put("personName", personName);
                map.put("serviceDesc", jobInformationInfo.getServiceDesc());
                map.put("intendedIndustriesN", intendedIndustriesN);
                map.put("intendedPosition", jobInformationInfo.getIntendedPosition());
                map.put("hopeSalary", jobInformationInfo.getHopeSalary());
                map.put("methodN", jobInformationInfo.getMethod());
                map.put("address", jobInformationInfo.getAddress());
                map.put("educationN", educationN);
                map.put("status", jobInformationInfo.getStatus());

                String statusN;
                if (jobInformationInfo.getStatus() != null && "1".equals(jobInformationInfo.getStatus())) {
                    statusN = "已审核";
                } else {
                    statusN = "未审核";
                }
                map.put("statusN", statusN);
                map.put("isReadNumber", isReadNumber);

                mapList.add(map);
            }

            if (exportFlag) {
                String filePath = exportXls(mapList);
                msgResult.setData(filePath);
            } else {
                Map<String, Object> dataMap = PojoUtils.pageWrapper(page);
                dataMap.put("data", mapList);
                msgResult.setData(dataMap);
            }

            msgResult.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;

    }


    private String exportXls(List<Map> mapList) {
        String downloadUrl = "";

        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        //表头
        Row rowTitle = sheet.createRow(0);

        String[] titles = new String[]{"序号", "求职人员", "服务描述",
                "意向行业", "意向岗位", "期望薪资", "结算方式", "居住地",
                "最高学历", "审核状态"};

        for (int i = 0; i < titles.length; i++) {
            Cell cell = rowTitle.createCell(i);
            cell.setCellValue(titles[i]);
        }

        for (int i = 0; i < mapList.size(); i++) {
            Map<String, Object> map = mapList.get(i);

            Row row = sheet.createRow(i + 1);

            int colIndex = 0;
            row.createCell(colIndex++).setCellValue(i + 1);
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("personName"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("serviceDesc"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("intendedIndustriesN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("intendedPosition"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("hopeSalary"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("methodN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("address"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("educationN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("statusN"), ""));

        }

        ByteArrayOutputStream output = new ByteArrayOutputStream();

        try {
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            downloadUrl = OSSUtil.upload(ossConfig, "jobInformationList", "求职信息列表.xls", input);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }

        return downloadUrl;
    }


    @ApiOperation(value = "审核信息")
    @PostMapping("check/{id}")
    public MessageResult<JobInformationInfo> check(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        try {

            JobInformationInfo jobInformationInfo = jobInformationInfoService.get(id);
            jobInformationInfo.setStatus("1");
            jobInformationInfo.setUpdateBy(subject);
            jobInformationInfo.setUpdateTime(new Date());

            int affectCount = jobInformationInfoService.update(jobInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库审核失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
