package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.NewsInfo;
import com.jpsoft.employment.modules.base.service.NewsInfoService;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/newsInfo")
public class NewsInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private NewsInfoService newsInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;
	
	
	@ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<NewsInfo> create(){
        MessageResult<NewsInfo> msgResult = new MessageResult<>();

        NewsInfo newsInfo = new NewsInfo();

        msgResult.setData(newsInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<NewsInfo> add(@RequestBody NewsInfo newsInfo,@RequestAttribute String subject){
        MessageResult<NewsInfo> msgResult = new MessageResult<>();

        try {
            newsInfo.setId(UUID.randomUUID().toString());
			newsInfo.setDelFlag(false);
            newsInfo.setCreateBy(subject);
            newsInfo.setCreateTime(new Date());
            newsInfo.setBrowseNumber(0);
            newsInfo.setStatus("0");

            int affectCount = newsInfoService.insert(newsInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(newsInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<NewsInfo> edit(@PathVariable("id") String id){
        MessageResult<NewsInfo> msgResult = new MessageResult<>();

        try {
            NewsInfo newsInfo = newsInfoService.get(id);

            if (newsInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(newsInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<NewsInfo> update(@RequestBody NewsInfo newsInfo,@RequestAttribute String subject){
        MessageResult<NewsInfo> msgResult = new MessageResult<>();

        try {
		    newsInfo.setUpdateBy(subject);
            newsInfo.setUpdateTime(new Date());
		
            int affectCount = newsInfoService.update(newsInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(newsInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<NewsInfo> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<NewsInfo> msgResult = new MessageResult<>();

        try {
			
			NewsInfo newsInfo = newsInfoService.get(id);
            newsInfo.setDelFlag(true);
            newsInfo.setUpdateBy(subject);
            newsInfo.setUpdateTime(new Date());
			
			int affectCount = newsInfoService.update(newsInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                NewsInfo newsInfo = newsInfoService.get(id);
                newsInfo.setDelFlag(true);
                newsInfo.setUpdateBy(subject);
                newsInfo.setUpdateTime(new Date());

                affectCount += newsInfoService.update(newsInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String title,String category,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(title)) {
            searchParams.put("title","%" + title + "%");
        }

        if (StringUtils.isNotEmpty(category)) {
            searchParams.put("category",category);
        }


        Page<NewsInfo> page = newsInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (NewsInfo newsInfo:page) {
            String categoryN = dataDictionaryService.findNameByCatalogNameAndValue("新闻类型",newsInfo.getCategory());
            if(StringUtils.isNotEmpty(categoryN)){
                newsInfo.setCategoryN(categoryN);
            }
        }


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="审核信息")
    @PostMapping("check/{id}")
    public MessageResult<NewsInfo> check(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<NewsInfo> msgResult = new MessageResult<>();

        try {

            NewsInfo newsInfo = newsInfoService.get(id);
            newsInfo.setStatus("1");
            newsInfo.setUpdateBy(subject);
            newsInfo.setUpdateTime(new Date());

            int affectCount = newsInfoService.update(newsInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库审核失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
