package com.jpsoft.employment.modules.base.controller;

import cn.hutool.core.util.IdcardUtil;
import com.github.pagehelper.Page;
import com.jpsoft.employment.config.OSSConfig;
import com.jpsoft.employment.modules.base.entity.EnterpriseInfo;
import com.jpsoft.employment.modules.base.entity.PersonLoginLog;
import com.jpsoft.employment.modules.base.service.EnterpriseInfoService;
import com.jpsoft.employment.modules.base.service.PersonLoginLogService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.DES3;
import com.jpsoft.employment.modules.common.utils.OSSUtil;
import com.jpsoft.employment.modules.common.utils.POIUtils;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.utils.IdCardUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.PersonInfo;
import com.jpsoft.employment.modules.base.service.PersonInfoService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/personInfo")
public class PersonInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;

    @Autowired
    private PersonLoginLogService personLoginLogService;

    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private OSSConfig ossConfig;


    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<PersonInfo> create() {
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        PersonInfo personInfo = new PersonInfo();

        msgResult.setData(personInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    public MessageResult<PersonInfo> add(@RequestBody PersonInfo personInfo, @RequestAttribute String subject) {
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            Date now = new Date();

            personInfo.setId(UUID.randomUUID().toString());
            personInfo.setDelFlag(false);
            personInfo.setCreateBy(subject);
            personInfo.setCreateTime(now);
            DES3 des3 = new DES3();
            personInfo.setPassword(des3.encrypt(jwtSecret, "123456"));
            personInfo.setStatus("0");
            personInfo.setApplicationCheckTime(now);

            int affectCount = personInfoService.insert(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PersonInfo> edit(@PathVariable("id") String id) {
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if (personInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    public MessageResult<PersonInfo> update(@RequestBody PersonInfo personInfo, @RequestAttribute String subject) {
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<PersonInfo> delete(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {

            PersonInfo personInfo = personInfoService.get(id);
            personInfo.setDelFlag(true);
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PersonInfo personInfo = personInfoService.get(id);
                personInfo.setDelFlag(true);
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());

                affectCount += personInfoService.update(personInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Object> pageList(
            String realName, String phone, String idCard, String status,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            @RequestParam(value = "exportFlag", defaultValue = "false") Boolean exportFlag,
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Object> msgResult = new MessageResult<>();

        try {

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (StringUtils.isNotEmpty(realName)) {
                searchParams.put("realName", realName + "%");
            }

            if (StringUtils.isNotEmpty(phone)) {
                searchParams.put("phone", phone + "%");
            }
            if (StringUtils.isNotEmpty(idCard)) {
                searchParams.put("idCard", idCard + "%");
            }
            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }

            Page<PersonInfo> page = personInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<Map> mapList = new ArrayList<>();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            for (PersonInfo personInfo : page) {
                Map map = new HashMap();
                map.put("id", personInfo.getId());
                map.put("phone", personInfo.getPhone());
                map.put("realName", personInfo.getRealName());
                map.put("idCard", personInfo.getIdCard());
                map.put("gender", personInfo.getGender());
                String genderN = "";
                if (StringUtils.isNotEmpty(personInfo.getGender())) {
                    if (personInfo.getGender().equals("0")) {
                        genderN = "女";
                    } else {
                        genderN = "男";
                    }
                }
                map.put("genderN", genderN);
                map.put("age", personInfo.getAge());

                String statusN;
                if ("1".equals(personInfo.getStatus())) {
                    statusN = "已认证";
                } else {
                    statusN = "未认证";
                }

                map.put("status", personInfo.getStatus());
                map.put("statusN", statusN);

                //最近登录时间
                String lastLoginTime = "";
                PersonLoginLog personLoginLog = personLoginLogService.lastLogin(personInfo.getId());
                if (personLoginLog != null) {
                    lastLoginTime = sdf.format(personLoginLog.getCreateTime());
                }

                map.put("lastLoginTime", lastLoginTime);


                //认证时间
                String checkTime = "";
                if (personInfo.getCheckTime() != null) {
                    checkTime = sdf.format(personInfo.getCheckTime());
                }

                map.put("checkTime", checkTime);

                //申请认证时间
                String applicationCheckTime = "";
                if (personInfo.getApplicationCheckTime() != null) {
                    applicationCheckTime = sdf.format(personInfo.getApplicationCheckTime());
                }

                map.put("applicationCheckTime", applicationCheckTime);

                mapList.add(map);
            }


            if (exportFlag) {
                String filePath = exportXls(mapList);
                msgResult.setData(filePath);
            } else {
                Map<String, Object> dataMap = PojoUtils.pageWrapper(page);
                dataMap.put("data", mapList);
                msgResult.setData(dataMap);
            }

            msgResult.setResult(true);

        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;

    }


    @ApiOperation(value = "导入人员")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadFile", value = "上传文件", required = true, paramType = "form", dataType = "__file")
    })
    public MessageResult<String> importXls(MultipartFile uploadFile, @RequestAttribute String subject) {
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 5;

            int count = sheet1.getLastRowNum();

            Date now = new Date();

            for (int rowIndex = 1; rowIndex <= count; rowIndex++) {
                try {

                    String userName = poiUtils.getCellValue(sheetIndex, rowIndex, 0).toString().replace(" ", "");

                    String realName = poiUtils.getCellValue(sheetIndex, rowIndex, 1).toString().replace(" ", "");

                    String phone = poiUtils.getCellValue(sheetIndex, rowIndex, 2).toString().replace(" ", "");

                    String idCard = poiUtils.getCellValue(sheetIndex, rowIndex, 3).toString().replace(" ", "");

                    String enterpriseName = poiUtils.getCellValue(sheetIndex, rowIndex, 4).toString().replace(" ", "");

                    if (StringUtils.isEmpty(userName)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写用户名!");
                        failCount++;
                        continue;
                    }


                    if (StringUtils.isEmpty(realName)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写真实姓名!");
                        failCount++;
                        continue;
                    }


                    if (StringUtils.isEmpty(phone)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写联系电话!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(idCard)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写身份证号!");
                        failCount++;
                        continue;
                    }

                    //用户身份
                    PersonInfo personInfo = new PersonInfo();
                    personInfo.setId(UUID.randomUUID().toString());
                    personInfo.setPhone(phone);
                    personInfo.setUserName(userName);
                    personInfo.setRealName(realName);
                    personInfo.setIdCard(idCard);
                    personInfo.setStatus("1");
                    personInfo.setCreateBy(subject);
                    personInfo.setCreateTime(now);
                    personInfo.setDelFlag(false);
                    personInfo.setIsAnonymous(true);
                    personInfo.setIntention("1");//求职中
                    personInfo.setJobStatus("1");//显示简历
                    personInfo.setCheckTime(now);
                    personInfo.setApplicationCheckTime(now);

                    //是否填写关联企业
                    if (StringUtils.isNotEmpty(enterpriseName)) {
                        EnterpriseInfo enterpriseInfo = enterpriseInfoService.findByName(enterpriseName);
                        if (enterpriseInfo != null) {
                            personInfo.setEnterpriseId(enterpriseInfo.getId());
                        } else {
                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("填写的企业名称在系统中不存在，请检查企业名称!");
                            failCount++;
                            continue;
                        }
                    }

                    int gender = IdcardUtil.getGenderByIdCard(personInfo.getIdCard());
                    int age = IdcardUtil.getAgeByIdCard(personInfo.getIdCard());

                    personInfo.setGender(String.valueOf(gender));
                    personInfo.setAge(age);


                    //保存新用户
                    personInfoService.insert(personInfo);
                    affectCount++;
                } catch (Exception innerEx) {
                    logger.error(innerEx.getMessage(), innerEx);
                }
            }

            if (failCount > 0) {
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex, validateColIndex, 1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig, "import", "error.xls", input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            } else {
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "审核信息")
    @PostMapping("check/{id}")
    public MessageResult<PersonInfo> check(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {

            Date now = new Date();
            PersonInfo personInfo = personInfoService.get(id);
            personInfo.setStatus("1");
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(now);
            personInfo.setCheckTime(now);

            if (StringUtils.isNotEmpty(personInfo.getIdCard())) {
                int gender = IdcardUtil.getGenderByIdCard(personInfo.getIdCard());
                int age = IdcardUtil.getAgeByIdCard(personInfo.getIdCard());

                personInfo.setGender(String.valueOf(gender));
                personInfo.setAge(age);
            }


            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库审核失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    private String exportXls(List<Map> mapList) {
        String downloadUrl = "";

        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        //表头
        Row rowTitle = sheet.createRow(0);

        String[] titles = new String[]{"序号", "姓名", "联系电话", "身份证号", "性别", "年龄", "最后登录时间", "认证状态", "认证时间"};

        for (int i = 0; i < titles.length; i++) {
            Cell cell = rowTitle.createCell(i);
            cell.setCellValue(titles[i]);
        }

        for (int i = 0; i < mapList.size(); i++) {
            Map<String, Object> map = mapList.get(i);

            Row row = sheet.createRow(i + 1);

            int colIndex = 0;
            row.createCell(colIndex++).setCellValue(i + 1);
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("realName"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("phone"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("idCard"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("genderN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("age"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("lastLoginTime"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("statusN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("applicationCheckTime"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("checkTime"), ""));
        }

        ByteArrayOutputStream output = new ByteArrayOutputStream();

        try {
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            downloadUrl = OSSUtil.upload(ossConfig, "personList", "人员列表.xls", input);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }

        return downloadUrl;
    }
}
