package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.base.entity.PersonLoginLog;
import com.jpsoft.employment.modules.base.service.PersonLoginLogService;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/personLoginLog")
public class PersonLoginLogController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PersonLoginLogService personLoginLogService;
	
	
	@ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PersonLoginLog> create(){
        MessageResult<PersonLoginLog> msgResult = new MessageResult<>();

        PersonLoginLog personLoginLog = new PersonLoginLog();

        msgResult.setData(personLoginLog);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PersonLoginLog> add(@RequestBody PersonLoginLog personLoginLog,@RequestAttribute String subject){
        MessageResult<PersonLoginLog> msgResult = new MessageResult<>();

        try {
            personLoginLog.setId(UUID.randomUUID().toString());
			personLoginLog.setDelFlag(false);
            personLoginLog.setCreateBy(subject);
            personLoginLog.setCreateTime(new Date());

            int affectCount = personLoginLogService.insert(personLoginLog);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personLoginLog);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PersonLoginLog> edit(@PathVariable("id") String id){
        MessageResult<PersonLoginLog> msgResult = new MessageResult<>();

        try {
            PersonLoginLog personLoginLog = personLoginLogService.get(id);

            if (personLoginLog != null) {
                msgResult.setResult(true);
                msgResult.setData(personLoginLog);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PersonLoginLog> update(@RequestBody PersonLoginLog personLoginLog,@RequestAttribute String subject){
        MessageResult<PersonLoginLog> msgResult = new MessageResult<>();

        try {
		    personLoginLog.setUpdateBy(subject);
            personLoginLog.setUpdateTime(new Date());
		
            int affectCount = personLoginLogService.update(personLoginLog);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personLoginLog);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<PersonLoginLog> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<PersonLoginLog> msgResult = new MessageResult<>();

        try {
			
			PersonLoginLog personLoginLog = personLoginLogService.get(id);
            personLoginLog.setDelFlag(true);
            personLoginLog.setUpdateBy(subject);
            personLoginLog.setUpdateTime(new Date());
			
			int affectCount = personLoginLogService.update(personLoginLog);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PersonLoginLog personLoginLog = personLoginLogService.get(id);
                personLoginLog.setDelFlag(true);
                personLoginLog.setUpdateBy(subject);
                personLoginLog.setUpdateTime(new Date());

                affectCount += personLoginLogService.update(personLoginLog);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }


        Page<PersonLoginLog> page = personLoginLogService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
