package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.config.OSSConfig;
import com.jpsoft.employment.modules.base.dto.ImageDTO;
import com.jpsoft.employment.modules.base.entity.EnterpriseInfo;
import com.jpsoft.employment.modules.base.service.EnterpriseInfoService;
import com.jpsoft.employment.modules.base.service.RecruitPersonRelationService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.OSSUtil;
import com.jpsoft.employment.modules.common.utils.POIUtils;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.RecruitInformationInfo;
import com.jpsoft.employment.modules.base.service.RecruitInformationInfoService;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/recruitInformationInfo")
public class RecruitInformationInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private RecruitInformationInfoService recruitInformationInfoService;

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private RecruitPersonRelationService recruitPersonRelationService;

    @Autowired
    private OSSConfig ossConfig;


    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<RecruitInformationInfo> create() {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        RecruitInformationInfo recruitInformationInfo = new RecruitInformationInfo();
        List<ImageDTO> imageDTOList = new ArrayList<>();
        recruitInformationInfo.setFileList(imageDTOList);

        msgResult.setData(recruitInformationInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    public MessageResult<RecruitInformationInfo> add(@RequestBody RecruitInformationInfo recruitInformationInfo, @RequestAttribute String subject) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {
            recruitInformationInfo.setId(UUID.randomUUID().toString());
            recruitInformationInfo.setDelFlag(false);
            recruitInformationInfo.setCreateBy(subject);
            recruitInformationInfo.setCreateTime(new Date());
            recruitInformationInfo.setUpdateTime(new Date());
            recruitInformationInfo.setStatus("0");
            recruitInformationInfo.setBrowseNumber(0);
            recruitInformationInfo.setIsOnline(true);
            recruitInformationInfo.setIsTopping(false);

            if (recruitInformationInfo.getFileList() != null) {
                JSONArray jsonArray = new JSONArray();

                for (ImageDTO dto : recruitInformationInfo.getFileList()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", dto.getName());
                    jsonObject.put("url", dto.getUrl());

                    jsonArray.add(jsonObject);
                }

                if (jsonArray.size() > 0) {
                    recruitInformationInfo.setImages(jsonArray.toString());
                }
            }

            int affectCount = recruitInformationInfoService.insert(recruitInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(recruitInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<RecruitInformationInfo> edit(@PathVariable("id") String id) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {
            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);

            if (recruitInformationInfo != null) {
                List<ImageDTO> imageDTOList = new ArrayList<>();
                if (StringUtils.isNotEmpty(recruitInformationInfo.getImages())) {
                    JSONArray jsonArray = JSONArray.fromObject(recruitInformationInfo.getImages());

                    for (Object object : jsonArray) {
                        JSONObject jsonObject = JSONObject.fromObject(object);
                        if (jsonObject != null) {
                            String name = jsonObject.get("name").toString();
                            String url = jsonObject.get("url").toString();

                            ImageDTO dto = new ImageDTO();
                            dto.setName(name);
                            dto.setUrl(url);
                            imageDTOList.add(dto);
                        }
                    }
                }

                recruitInformationInfo.setFileList(imageDTOList);

                msgResult.setResult(true);
                msgResult.setData(recruitInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    public MessageResult<RecruitInformationInfo> update(@RequestBody RecruitInformationInfo recruitInformationInfo, @RequestAttribute String subject) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());

            if (recruitInformationInfo.getFileList() != null) {
                JSONArray jsonArray = new JSONArray();

                for (ImageDTO dto : recruitInformationInfo.getFileList()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", dto.getName());
                    jsonObject.put("url", dto.getUrl());

                    jsonArray.add(jsonObject);
                }

                if (jsonArray.size() > 0) {
                    recruitInformationInfo.setImages(jsonArray.toString());
                }
            }

            int affectCount = recruitInformationInfoService.update(recruitInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(recruitInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<RecruitInformationInfo> delete(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            recruitInformationInfo.setDelFlag(true);
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());

            int affectCount = recruitInformationInfoService.update(recruitInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
                recruitInformationInfo.setDelFlag(true);
                recruitInformationInfo.setUpdateBy(subject);
                recruitInformationInfo.setUpdateTime(new Date());

                affectCount += recruitInformationInfoService.update(recruitInformationInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Object> pageList(
            String name, String positionName, String intendedIndustries, String method,
            String status, String workArea, String isOnline, String isTopping,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            @RequestParam(value = "exportFlag", defaultValue = "false") Boolean exportFlag,
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Object> msgResult = new MessageResult<>();

        try {

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time", "desc"));

            if (StringUtils.isNotEmpty(name)) {
                searchParams.put("name", "%" + name + "%");
            }

            if (StringUtils.isNotEmpty(positionName)) {
                searchParams.put("positionName", positionName + "%");
            }

            if (StringUtils.isNotEmpty(workArea)) {
                searchParams.put("workArea", workArea + "%");
            }

            if (StringUtils.isNotEmpty(intendedIndustries)) {
                searchParams.put("intendedIndustries", intendedIndustries);
            }

            if (StringUtils.isNotEmpty(method)) {
                searchParams.put("method", method);
            }

            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }

            if (StringUtils.isNotEmpty(isOnline)) {
                if ("0".equals(isOnline)) {
                    searchParams.put("isOnline", false);
                } else {
                    searchParams.put("isOnline", true);
                }

            }

            if (StringUtils.isNotEmpty(isTopping)) {
                if ("0".equals(isTopping)) {
                    searchParams.put("isTopping", false);
                } else {
                    searchParams.put("isTopping", true);
                }
            }


            Page<RecruitInformationInfo> page = recruitInformationInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<Map> mapList = new ArrayList<>();

            for (RecruitInformationInfo recruitInformationInfo : page) {
                Map map = new HashMap();

                String enterpriseName = "";
                EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(recruitInformationInfo.getEnterpriseId());
                if (enterpriseInfo != null) {
                    enterpriseName = enterpriseInfo.getName();
                }

                String industryN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业", recruitInformationInfo.getIndustry());
                if (StringUtils.isNotEmpty(industryN)) {
                    recruitInformationInfo.setIndustryN(industryN);
                }

                String settlementMethodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式", recruitInformationInfo.getSettlementMethod());
                if (StringUtils.isNotEmpty(settlementMethodN)) {
                    recruitInformationInfo.setSettlementMethodN(settlementMethodN);
                }

                Integer isReadNumber = recruitPersonRelationService.isReadCountByRecruit(recruitInformationInfo.getId());

                recruitInformationInfo.setIsReadNumber(isReadNumber);


                map.put("id", recruitInformationInfo.getId());
                map.put("enterpriseName", enterpriseName);
                map.put("industryN", industryN);
                map.put("settlementMethodN", settlementMethodN);
                map.put("positionName", recruitInformationInfo.getPositionName());
                map.put("salary", recruitInformationInfo.getSalary());
                map.put("recruitingNumbers", recruitInformationInfo.getRecruitingNumbers());
                map.put("contacts", recruitInformationInfo.getContacts());
                map.put("contactsPhone", recruitInformationInfo.getContactsPhone());
                map.put("workArea", recruitInformationInfo.getWorkArea());
                map.put("address", recruitInformationInfo.getAddress());
                map.put("browseNumber", recruitInformationInfo.getBrowseNumber());
                map.put("ageRequirement", recruitInformationInfo.getAgeRequirement());
                map.put("isTopping", recruitInformationInfo.getIsTopping());

                String statusN;
                if ("1".equals(recruitInformationInfo.getStatus())) {
                    statusN = "已审核";
                } else {
                    statusN = "未审核";
                }
                map.put("status", recruitInformationInfo.getStatus());
                map.put("statusN", statusN);

                String isOnlineN;
                if (recruitInformationInfo.getIsOnline()) {
                    isOnlineN = "上架";
                } else {
                    isOnlineN = "下架";
                }
                map.put("isOnline", recruitInformationInfo.getIsOnline());
                map.put("isOnlineN", isOnlineN);
                map.put("isReadNumber", isReadNumber);
                mapList.add(map);

            }


            if (exportFlag) {
                String filePath = exportXls(mapList);
                msgResult.setData(filePath);
            } else {
                Map<String, Object> dataMap = PojoUtils.pageWrapper(page);
                dataMap.put("data", mapList);
                msgResult.setData(dataMap);
            }

            msgResult.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;
    }


    private String exportXls(List<Map> mapList) {
        String downloadUrl = "";

        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        //表头
        Row rowTitle = sheet.createRow(0);

        String[] titles = new String[]{"序号", "企业名称", "行业",
                "职位名称", "薪资待遇", "结算方式", "年龄要求", "招聘人数",
                "联系人", "联系电话", "工作地区", "详细地址", "浏览次数", "审核状态", "是否上架"};

        for (int i = 0; i < titles.length; i++) {
            Cell cell = rowTitle.createCell(i);
            cell.setCellValue(titles[i]);
        }

        for (int i = 0; i < mapList.size(); i++) {
            Map<String, Object> map = mapList.get(i);

            Row row = sheet.createRow(i + 1);

            int colIndex = 0;
            row.createCell(colIndex++).setCellValue(i + 1);
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("enterpriseName"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("industryN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("positionName"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("salary"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("settlementMethodN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("ageRequirement"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("recruitingNumbers"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("contacts"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("contactsPhone"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("workArea"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("address"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("browseNumber"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("statusN"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("isOnlineN"), ""));

        }

        ByteArrayOutputStream output = new ByteArrayOutputStream();

        try {
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            downloadUrl = OSSUtil.upload(ossConfig, "RecruitInfoList", "招聘信息列表.xls", input);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }

        return downloadUrl;
    }


    @ApiOperation(value = "审核信息")
    @PostMapping("check/{id}")
    public MessageResult<RecruitInformationInfo> check(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            recruitInformationInfo.setStatus("1");
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());

            int affectCount = recruitInformationInfoService.update(recruitInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库审核失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "修改是否在线")
    @PostMapping("changeIsOnline/{id}")
    public MessageResult<RecruitInformationInfo> changeIsOnline(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());

            if (recruitInformationInfo.getIsOnline()) {
                recruitInformationInfo.setIsOnline(false);
            } else {
                recruitInformationInfo.setIsOnline(true);
            }

            int affectCount = recruitInformationInfoService.update(recruitInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库修改失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "修改是否置顶")
    @PostMapping("changeTopping/{id}")
    public MessageResult<RecruitInformationInfo> changeTopping(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<RecruitInformationInfo> msgResult = new MessageResult<>();

        try {

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());

            if (recruitInformationInfo.getIsTopping() != null && recruitInformationInfo.getIsTopping()) {
                recruitInformationInfo.setIsTopping(false);
            } else {
                recruitInformationInfo.setIsTopping(true);
            }

            int affectCount = recruitInformationInfoService.update(recruitInformationInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库修改失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "导入招聘信息")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadFile", value = "上传文件", required = true, paramType = "form", dataType = "__file")
    })
    public MessageResult<String> importXls(MultipartFile uploadFile, @RequestAttribute String subject) {
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 12;

            int count = sheet1.getLastRowNum();

            for (int rowIndex = 1; rowIndex <= count; rowIndex++) {
                try {

                    String enterpriseName = poiUtils.getCellValue(sheetIndex, rowIndex, 0).toString().replace(" ", "");
                    String industry = poiUtils.getCellValue(sheetIndex, rowIndex, 1).toString().replace(" ", "");
                    String positionName = poiUtils.getCellValue(sheetIndex, rowIndex, 2).toString().replace(" ", "");
                    String settlementMethod = poiUtils.getCellValue(sheetIndex, rowIndex, 3).toString().replace(" ", "");
                    String salary = poiUtils.getCellValue(sheetIndex, rowIndex, 4).toString().replace(" ", "");
                    String recruitingNumbers = poiUtils.getCellValue(sheetIndex, rowIndex, 5).toString().replace(" ", "");
                    String desc = poiUtils.getCellValue(sheetIndex, rowIndex, 6).toString().replace(" ", "");
                    String contacts = poiUtils.getCellValue(sheetIndex, rowIndex, 7).toString().replace(" ", "");
                    String contactsPhone = poiUtils.getCellValue(sheetIndex, rowIndex, 8).toString().replace(" ", "");
                    String workArea = poiUtils.getCellValue(sheetIndex, rowIndex, 9).toString().replace(" ", "");
                    String address = poiUtils.getCellValue(sheetIndex, rowIndex, 10).toString().replace(" ", "");
                    String ageRequirement = poiUtils.getCellValue(sheetIndex, rowIndex, 11).toString().replace(" ", "");

                    if (StringUtils.isEmpty(enterpriseName)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写企业名称!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(industry)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写行业!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(positionName)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写职位名称!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(settlementMethod)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写结算方式!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(salary)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写薪资待遇!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(recruitingNumbers)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写招聘人数!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(desc)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写工作描述!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(contacts)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写联系人!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(contactsPhone)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写联系电话!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(workArea)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写工作地区!");
                        failCount++;
                        continue;
                    }

                    if (StringUtils.isEmpty(address)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("未填写详细地址!");
                        failCount++;
                        continue;
                    }


                    EnterpriseInfo enterpriseInfo = enterpriseInfoService.findByName(enterpriseName);
                    if (enterpriseInfo == null) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("填写的企业名称在系统中不存在，请检查企业名称!");
                        failCount++;
                        continue;
                    }

                    String industryValue = dataDictionaryService.findValueByCatalogNameAndName("意向行业", industry);
                    if (StringUtils.isEmpty(industryValue)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("填写的行业在系统中不存在，请检查行业!");
                        failCount++;
                        continue;
                    }

                    String settlementMethodValue = dataDictionaryService.findValueByCatalogNameAndName("结算方式", settlementMethod);
                    if (StringUtils.isEmpty(settlementMethodValue)) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("填写的结算方式在系统中不存在，请检查结算方式!");
                        failCount++;
                        continue;
                    }

                    //企业
                    RecruitInformationInfo recruitInformationInfo = new RecruitInformationInfo();
                    recruitInformationInfo.setId(UUID.randomUUID().toString());
                    recruitInformationInfo.setDelFlag(false);
                    recruitInformationInfo.setCreateBy(subject);
                    recruitInformationInfo.setCreateTime(new Date());
                    recruitInformationInfo.setUpdateTime(new Date());
                    recruitInformationInfo.setStatus("1");
                    recruitInformationInfo.setBrowseNumber(0);
                    recruitInformationInfo.setEnterpriseId(enterpriseInfo.getId());
                    recruitInformationInfo.setIndustry(industryValue);
                    recruitInformationInfo.setPositionName(positionName);
                    recruitInformationInfo.setSettlementMethod(settlementMethodValue);
                    recruitInformationInfo.setSalary(salary);
                    recruitInformationInfo.setRecruitingNumbers(recruitingNumbers);
                    recruitInformationInfo.setDesc(desc);
                    recruitInformationInfo.setContacts(contacts);
                    recruitInformationInfo.setContactsPhone(contactsPhone);
                    recruitInformationInfo.setWorkArea(workArea);
                    recruitInformationInfo.setAddress(address);
                    recruitInformationInfo.setIsOnline(true);
                    recruitInformationInfo.setIsTopping(false);
                    recruitInformationInfo.setAgeRequirement(ageRequirement);


                    //保存新企业
                    recruitInformationInfoService.insert(recruitInformationInfo);
                    affectCount++;
                } catch (Exception innerEx) {
                    logger.error(innerEx.getMessage(), innerEx);
                }
            }

            if (failCount > 0) {
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex, validateColIndex, 1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig, "import", "error.xls", input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            } else {
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
