package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.base.entity.JobInformationInfo;
import com.jpsoft.employment.modules.base.entity.PersonInfo;
import com.jpsoft.employment.modules.base.entity.RecruitInformationInfo;
import com.jpsoft.employment.modules.base.service.JobInformationInfoService;
import com.jpsoft.employment.modules.base.service.PersonInfoService;
import com.jpsoft.employment.modules.base.service.RecruitInformationInfoService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.RecruitPersonRelation;
import com.jpsoft.employment.modules.base.service.RecruitPersonRelationService;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/recruitPersonRelation")
public class RecruitPersonRelationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private RecruitPersonRelationService recruitPersonRelationService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private JobInformationInfoService jobInformationInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;


    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<RecruitPersonRelation> create() {
        MessageResult<RecruitPersonRelation> msgResult = new MessageResult<>();

        RecruitPersonRelation recruitPersonRelation = new RecruitPersonRelation();

        msgResult.setData(recruitPersonRelation);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    public MessageResult<RecruitPersonRelation> add(@RequestBody RecruitPersonRelation recruitPersonRelation, @RequestAttribute String subject) {
        MessageResult<RecruitPersonRelation> msgResult = new MessageResult<>();

        try {
            recruitPersonRelation.setId(UUID.randomUUID().toString());
            recruitPersonRelation.setDelFlag(false);
            recruitPersonRelation.setCreateBy(subject);
            recruitPersonRelation.setCreateTime(new Date());

            int affectCount = recruitPersonRelationService.insert(recruitPersonRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(recruitPersonRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<RecruitPersonRelation> edit(@PathVariable("id") String id) {
        MessageResult<RecruitPersonRelation> msgResult = new MessageResult<>();

        try {
            RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.get(id);

            if (recruitPersonRelation != null) {
                msgResult.setResult(true);
                msgResult.setData(recruitPersonRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    public MessageResult<RecruitPersonRelation> update(@RequestBody RecruitPersonRelation recruitPersonRelation, @RequestAttribute String subject) {
        MessageResult<RecruitPersonRelation> msgResult = new MessageResult<>();

        try {
            recruitPersonRelation.setUpdateBy(subject);
            recruitPersonRelation.setUpdateTime(new Date());

            int affectCount = recruitPersonRelationService.update(recruitPersonRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(recruitPersonRelation);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<RecruitPersonRelation> delete(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<RecruitPersonRelation> msgResult = new MessageResult<>();

        try {

            RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.get(id);
            recruitPersonRelation.setDelFlag(true);
            recruitPersonRelation.setUpdateBy(subject);
            recruitPersonRelation.setUpdateTime(new Date());

            int affectCount = recruitPersonRelationService.update(recruitPersonRelation);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.get(id);
                recruitPersonRelation.setDelFlag(true);
                recruitPersonRelation.setUpdateBy(subject);
                recruitPersonRelation.setUpdateTime(new Date());

                affectCount += recruitPersonRelationService.update(recruitPersonRelation);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String informationId, String name, String phone,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String, Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time", "desc"));

        if (StringUtils.isNotEmpty(informationId)) {
            searchParams.put("informationId", informationId);
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name", name + "%");
        }

        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone", phone + "%");
        }


        Page<RecruitPersonRelation> page = recruitPersonRelationService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

        for (RecruitPersonRelation recruitPersonRelation:page) {
            PersonInfo personInfo = personInfoService.get(recruitPersonRelation.getWorkPersonId());
            if(personInfo!=null){
                recruitPersonRelation.setWorkPersonName(personInfo.getRealName());
                recruitPersonRelation.setPhone(personInfo.getPhone());
                recruitPersonRelation.setIdCard(personInfo.getIdCard());
                recruitPersonRelation.setAge(personInfo.getAge());
                recruitPersonRelation.setGender(personInfo.getGender());
            }
        }


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="获取信息")
    @GetMapping("jobInformationDetail/{id}")
    public MessageResult<JobInformationInfo> jobInformationDetail(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<JobInformationInfo> msgResult = new MessageResult<>();

        try {
            RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.get(id);


            JobInformationInfo jobInformationInfo = jobInformationInfoService.get(recruitPersonRelation.getJobInformationId());

            if (jobInformationInfo != null) {
                //修改是否读取状态
                if(recruitPersonRelation.getIsRead()==null||!recruitPersonRelation.getIsRead()){
                    recruitPersonRelation.setIsRead(true);
                    recruitPersonRelation.setUpdateBy(subject);
                    recruitPersonRelation.setUpdateTime(new Date());
                    recruitPersonRelationService.update(recruitPersonRelation);

                    jobInformationInfo.setIsReadStatus(true);
                }


                PersonInfo personInfo = personInfoService.get(jobInformationInfo.getPersonId());
                if(personInfo!=null){
                    jobInformationInfo.setPersonName(personInfo.getRealName());
                    jobInformationInfo.setPersonPhone(personInfo.getPhone());
                }

                String intendedIndustriesN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业",jobInformationInfo.getIntendedIndustries());
                if(StringUtils.isNotEmpty(intendedIndustriesN)){
                    jobInformationInfo.setIntendedIndustriesN(intendedIndustriesN);
                }

                String methodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式",jobInformationInfo.getMethod());
                if(StringUtils.isNotEmpty(methodN)){
                    jobInformationInfo.setMethodN(methodN);
                }

                String workExperienceN = dataDictionaryService.findNameByCatalogNameAndValue("工作经验",jobInformationInfo.getWorkExperience());
                if(StringUtils.isNotEmpty(workExperienceN)){
                    jobInformationInfo.setWorkExperienceN(workExperienceN);
                }


                String educationN = dataDictionaryService.findNameByCatalogNameAndValue("学历",jobInformationInfo.getEducation());
                if(StringUtils.isNotEmpty(educationN)){
                    jobInformationInfo.setEducationN(educationN);
                }

                jobInformationInfo.setIsRead(true);

                msgResult.setResult(true);
                msgResult.setData(jobInformationInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
