package com.jpsoft.employment.modules.base.controller;


import com.github.pagehelper.Page;
import com.jpsoft.employment.config.OSSConfig;
import com.jpsoft.employment.modules.base.entity.GenerateReport;
import com.jpsoft.employment.modules.base.entity.JobInformationInfo;
import com.jpsoft.employment.modules.base.entity.RecruitInformationInfo;
import com.jpsoft.employment.modules.base.entity.RecruitPersonRelation;
import com.jpsoft.employment.modules.base.service.*;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.common.utils.OSSUtil;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/statistical/report")
public class ReportController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PersonLoginLogService personLoginLogService;

    @Autowired
    private RecruitPersonRelationService recruitPersonRelationService;

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private RecruitInformationInfoService recruitInformationInfoService;

    @Autowired
    private JobInformationInfoService jobInformationInfoService;

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private GenerateReportService generateReportService;


    @ApiOperation(value = "日报表")
    @RequestMapping(value = "dailyReport", method = RequestMethod.POST)
    public MessageResult<Object> dailyReport(String dateRange, @RequestParam(value = "exportFlag", defaultValue = "false") Boolean exportFlag) {

        MessageResult<Object> msgResult = new MessageResult<>();

        try {

            Map<String, Object> searchParams = new HashMap<>();

            Map<String, Object> searchParams1 = new HashMap<>();

            if (StringUtils.isNotEmpty(dateRange)) {
                String[] date = dateRange.split(",");
                String startTime = date[0] + " 00:00:00";
                String endTime = date[1] + " 23:59:59";
                searchParams.put("startTime", startTime);
                searchParams.put("endTime", endTime);


                searchParams1.put("endTime", endTime);
            }


            List<Map> mapList = new ArrayList<>();

            Map map = new HashMap();

            //登录人数，新增人数
            Integer loginNum = personLoginLogService.dailyReportList(searchParams);

            //登录人数，累计人数
            List<String> personIdList = personLoginLogService.totalDailyReportList(searchParams1);
            Integer totalLoginNum = personIdList.size();

            //今日注册人数
            Integer dailyPersonRegNum = personInfoService.dailyPersonReg(searchParams);

            //累计注册人数
            Integer totalPersonRegNum = personInfoService.dailyPersonReg(searchParams1);

            //认证人数，新增人数
            Integer personNum = personInfoService.dailyReportList(searchParams);

            //认证人数，累计人数
            Integer totalPersonNum = personInfoService.dailyReportList(searchParams1);

            //简历数 累计简历数
            Integer totalJobInformation = jobInformationInfoService.countList(searchParams1);

            //认证企业数，新增数
            Integer enterpriseNum = enterpriseInfoService.dailyReportList(searchParams);

            //认证企业数，累计数
            Integer totalEnterpriseNum = enterpriseInfoService.dailyReportList(searchParams1);

            //累计岗位数量
            Integer totalRecruitStatusNum = recruitInformationInfoService.dailyReportList(searchParams1);

            //累计岗位需求人数
            Integer totalJobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);

            //新增岗位数量
            searchParams.put("status", "1");
            searchParams.put("isOnline", true);
            searchParams.put("delFlag", false);
            Integer recruitNum = recruitInformationInfoService.dailyReportList(searchParams);

            //实时岗位数量
            searchParams1.put("status", "1");
            searchParams1.put("isOnline", true);
            searchParams1.put("delFlag", false);

            //岗位需求人数
            Integer jobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);


            Integer totalRecruitNum = recruitInformationInfoService.dailyReportList(searchParams1);

            //今日报名人数
            searchParams = new HashMap<>();

            if (StringUtils.isNotEmpty(dateRange)) {
                String[] date = dateRange.split(",");
                String startTime = date + " 00:00:00";
                String endTime = date + " 23:59:59";
                searchParams.put("startTime", startTime);
                searchParams.put("endTime", endTime);
            }
            Integer jobInformationNum = recruitPersonRelationService.dailyReportList(searchParams);

            map.put("loginNum", loginNum);
            map.put("totalLoginNum", totalLoginNum);

            map.put("personNum", personNum);
            map.put("totalPersonNum", totalPersonNum);

            map.put("totalJobInformation", totalJobInformation);

            map.put("dailyPersonRegNum", dailyPersonRegNum);
            map.put("totalPersonRegNum", totalPersonRegNum);

            map.put("enterpriseNum", enterpriseNum);
            map.put("totalEnterpriseNum", totalEnterpriseNum);

            map.put("recruitNum", recruitNum);
            map.put("totalRecruitNum", totalRecruitNum);
            map.put("totalRecruitStatusNum", totalRecruitStatusNum);

            map.put("jobInformationNum", jobInformationNum);

            map.put("jobRequirementNum", jobRequirementNum);

            map.put("totalJobRequirementNum", totalJobRequirementNum);

            mapList.add(map);

            if (exportFlag) {
                String filePath = exportXls(mapList);
                msgResult.setData(filePath);
            } else {
                Map<String, Object> dataMap = new HashMap<>();
                dataMap.put("data", mapList);
                msgResult.setData(dataMap);
            }

            msgResult.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;
    }


    private String exportXls(List<Map> mapList) {
        String downloadUrl = "";

        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        //表头
        Row rowTitle = sheet.createRow(0);

        String[] titles = new String[]{"序号", "新增登录人数", "累计登录人数", "今日注册人数", "累计注册人数", "新增求职登记人数", "累计求职登记人数", "累计简历数",
                "新增线上认证企业数量", "累计认证企业数量", "新增岗位发布数量", "实时发布岗位数", "累计发布岗位数", "岗位需求人数", "累计岗位需求人数", "今日报名人数"};

        for (int i = 0; i < titles.length; i++) {
            Cell cell = rowTitle.createCell(i);
            cell.setCellValue(titles[i]);
        }

        for (int i = 0; i < mapList.size(); i++) {
            Map<String, Object> map = mapList.get(i);

            Row row = sheet.createRow(i + 1);

            int colIndex = 0;
            row.createCell(colIndex++).setCellValue(i + 1);
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("loginNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalLoginNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("dailyPersonRegNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalPersonRegNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("personNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalPersonNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalJobInformation"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("enterpriseNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalEnterpriseNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("recruitNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalRecruitNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalRecruitStatusNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("jobRequirementNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalJobRequirementNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("jobInformationNum"), ""));
        }

        ByteArrayOutputStream output = new ByteArrayOutputStream();

        try {
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            downloadUrl = OSSUtil.upload(ossConfig, "personList", "统计日报表.xls", input);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }

        return downloadUrl;
    }


    @ApiOperation(value = "分组日报表")
    @RequestMapping(value = "dailyGroupReport", method = RequestMethod.POST)
    public MessageResult<Object> dailyGroupReport(String dateRange, @RequestParam(value = "exportFlag", defaultValue = "false") Boolean exportFlag) {

        MessageResult<Object> msgResult = new MessageResult<>();

        try {
            String[] dateArr = dateRange.split(",");

            List<String> dayList = getTwoDaysDayDes(dateArr[0], dateArr[1]);

            List<Map> mapList = new ArrayList<>();

            for (String date : dayList) {
                Map<String, Object> searchParams = new HashMap<>();

                Map<String, Object> searchParams1 = new HashMap<>();

                String startTime = date + " 00:00:00";
                String endTime = date + " 23:59:59";
                searchParams.put("startTime", startTime);
                searchParams.put("endTime", endTime);


                searchParams1.put("endTime", endTime);

                Map map = new HashMap();
                map.put("date", date);

                //登录人数，新增人数
                Integer loginNum = personLoginLogService.dailyReportList(searchParams);

                //登录人数，累计人数
                List<String> personIdList = personLoginLogService.totalDailyReportList(searchParams1);
                Integer totalLoginNum = personIdList.size();

                //今日注册人数
                Integer dailyPersonRegNum = personInfoService.dailyPersonReg(searchParams);

                //累计注册人数
                Integer totalPersonRegNum = personInfoService.dailyPersonReg(searchParams1);

                //认证人数，新增人数
                Integer personNum = personInfoService.dailyReportList(searchParams);

                //认证人数，累计人数
                Integer totalPersonNum = personInfoService.dailyReportList(searchParams1);

                //简历数 累计简历数
                Integer totalJobInformation = jobInformationInfoService.countList(searchParams1);

                //认证企业数，新增数
                Integer enterpriseNum = enterpriseInfoService.dailyReportList(searchParams);

                //认证企业数，累计数
                Integer totalEnterpriseNum = enterpriseInfoService.dailyReportList(searchParams1);

                //累计岗位数量
                Integer totalRecruitStatusNum = recruitInformationInfoService.dailyReportList(searchParams1);

                //累计岗位需求人数
                Integer totalJobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);

                //新增岗位数量
                searchParams.put("status", "1");
                searchParams.put("isOnline", true);
                searchParams.put("delFlag", false);
                Integer recruitNum = recruitInformationInfoService.dailyReportList(searchParams);

                //实时岗位数量
                searchParams1.put("status", "1");
                searchParams1.put("isOnline", true);
                searchParams1.put("delFlag", false);

                //岗位需求人数
                Integer jobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);


                Integer totalRecruitNum = recruitInformationInfoService.dailyReportList(searchParams1);

                //今日报名人数
                searchParams = new HashMap<>();

                if (StringUtils.isNotEmpty(date)) {
                    String startTime2 = date + " 00:00:00";
                    String endTime2 = date + " 23:59:59";
                    searchParams.put("startTime", startTime2);
                    searchParams.put("endTime", endTime2);
                }
                Integer jobInformationNum = recruitPersonRelationService.dailyReportList(searchParams);

                map.put("loginNum", loginNum);
                map.put("totalLoginNum", totalLoginNum);

                map.put("personNum", personNum);
                map.put("totalPersonNum", totalPersonNum);

                map.put("totalJobInformation", totalJobInformation);

                map.put("dailyPersonRegNum", dailyPersonRegNum);
                map.put("totalPersonRegNum", totalPersonRegNum);

                map.put("enterpriseNum", enterpriseNum);
                map.put("totalEnterpriseNum", totalEnterpriseNum);

                map.put("recruitNum", recruitNum);
                map.put("totalRecruitNum", totalRecruitNum);
                map.put("totalRecruitStatusNum", totalRecruitStatusNum);

                map.put("jobInformationNum", jobInformationNum);

                map.put("jobRequirementNum", jobRequirementNum);

                map.put("totalJobRequirementNum", totalJobRequirementNum);

                mapList.add(map);
            }


            if (exportFlag) {
                String filePath = groupExportXls(mapList);
                msgResult.setData(filePath);
            } else {
                Map<String, Object> dataMap = new HashMap<>();
                dataMap.put("data", mapList);
                msgResult.setData(dataMap);
            }

            msgResult.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;
    }


    private String groupExportXls(List<Map> mapList) {
        String downloadUrl = "";

        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        //表头
        Row rowTitle = sheet.createRow(0);

        String[] titles = new String[]{"序号", "日期", "新增登录人数", "累计登录人数", "今日注册人数", "累计注册人数", "新增求职登记人数", "累计求职登记人数", "累计简历数",
                "新增线上认证企业数量", "累计认证企业数量", "新增岗位发布数量", "实时发布岗位数", "累计发布岗位数", "岗位需求人数", "累计岗位需求人数", "今日报名人数"};

        for (int i = 0; i < titles.length; i++) {
            Cell cell = rowTitle.createCell(i);
            cell.setCellValue(titles[i]);
        }

        for (int i = 0; i < mapList.size(); i++) {
            Map<String, Object> map = mapList.get(i);

            Row row = sheet.createRow(i + 1);

            int colIndex = 0;
            row.createCell(colIndex++).setCellValue(i + 1);
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("date"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("loginNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalLoginNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("dailyPersonRegNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalPersonRegNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("personNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalPersonNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalJobInformation"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("enterpriseNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalEnterpriseNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("recruitNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalRecruitNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalRecruitStatusNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("jobRequirementNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("totalJobRequirementNum"), ""));
            row.createCell(colIndex++).setCellValue(com.jpsoft.employment.modules.common.utils.StringUtils.strValue(map.get("jobInformationNum"), ""));
        }

        ByteArrayOutputStream output = new ByteArrayOutputStream();

        try {
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            downloadUrl = OSSUtil.upload(ossConfig, "personList", "分组统计日报表.xls", input);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }

        return downloadUrl;
    }


    /**
     * 根据一段时间获取该段时间的所有日期  正序排序
     *
     * @param startDate
     * @param endDate
     * @return
     */
    public List<String> getTwoDaysDayDes(String startDate, String endDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<String> dateList = new ArrayList<>();
        try {
            Date dateOne = sdf.parse(startDate);
            Date dateTwo = sdf.parse(endDate);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(dateOne);
            dateList.add(startDate);
            while (dateTwo.after(calendar.getTime())) {
                calendar.add(Calendar.DAY_OF_MONTH, 1);
                dateList.add(sdf.format(calendar.getTime()));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateList;
    }


    /**
     * 根据一段时间获取该段时间的所有日期  倒序排序
     *
     * @param startDate
     * @param endDate
     * @return yyyy-MM-dd
     */
    public static List<String> getTwoDaysDay(String startDate, String endDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<String> dateList = new ArrayList<>();
        try {
            Date dateOne = sdf.parse(startDate);
            Date dateTwo = sdf.parse(endDate);

            Calendar calendar = Calendar.getInstance();
            calendar.setTime(dateTwo);

            dateList.add(endDate);
            while (calendar.getTime().after(dateOne)) { //倒序时间,顺序after改before其他相应的改动。
                calendar.add(Calendar.DAY_OF_MONTH, -1);
                dateList.add(sdf.format(calendar.getTime()));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateList;
    }


    @ApiOperation(value = "分组日报表测试")
    @RequestMapping(value = "dailyGroupReportTest", method = RequestMethod.POST)
    public void dailyGroupReportTest() {
        try {
            Date now = new Date();

            Calendar cal = Calendar.getInstance();
            cal.setTime(now);
            cal.add(Calendar.DATE, -1);
            Date yesterday = cal.getTime();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String date = sdf.format(yesterday);

            Map<String, Object> searchParams = new HashMap<>();

            Map<String, Object> searchParams1 = new HashMap<>();

            if (StringUtils.isNotEmpty(date)) {
                String startTime = date + " 00:00:00";
                String endTime = date + " 23:59:59";
                searchParams.put("startTime", startTime);
                searchParams.put("endTime", endTime);


                searchParams1.put("endTime", endTime);
            }

            GenerateReport currGenerateReport = generateReportService.findByDate(searchParams);

            if (currGenerateReport == null) {

                //登录人数，新增人数
                Integer loginNum = personLoginLogService.dailyReportList(searchParams);

                //登录人数，累计人数
                List<String> personIdList = personLoginLogService.totalDailyReportList(searchParams1);
                Integer totalLoginNum = personIdList.size();

                //今日注册人数
                Integer dailyPersonRegNum = personInfoService.dailyPersonReg(searchParams);

                //累计注册人数
                Integer totalPersonRegNum = personInfoService.dailyPersonReg(searchParams1);

                //认证人数，新增人数
                Integer personNum = personInfoService.dailyReportList(searchParams);

                //认证人数，累计人数
                Integer totalPersonNum = personInfoService.dailyReportList(searchParams1);

                //简历数 累计简历数
                Integer totalJobInformation = jobInformationInfoService.countList(searchParams1);

                //认证企业数，新增数
                Integer enterpriseNum = enterpriseInfoService.dailyReportList(searchParams);

                //认证企业数，累计数
                Integer totalEnterpriseNum = enterpriseInfoService.dailyReportList(searchParams1);

                //累计岗位数量
                Integer totalRecruitStatusNum = recruitInformationInfoService.dailyReportList(searchParams1);

                //累计岗位需求人数
                Integer totalJobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);

                //新增岗位数量
                searchParams.put("status", "1");
                searchParams.put("isOnline", true);
                searchParams.put("delFlag", false);
                Integer recruitNum = recruitInformationInfoService.dailyReportList(searchParams);

                //实时岗位数量
                searchParams1.put("status", "1");
                searchParams1.put("isOnline", true);
                searchParams1.put("delFlag", false);

                //岗位需求人数
                Integer jobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);


                Integer totalRecruitNum = recruitInformationInfoService.dailyReportList(searchParams1);

                //今日报名人数
                searchParams = new HashMap<>();

                if (StringUtils.isNotEmpty(date)) {
                    String startTime = date + " 00:00:00";
                    String endTime = date + " 23:59:59";
                    searchParams.put("startTime", startTime);
                    searchParams.put("endTime", endTime);
                }
                Integer jobInformationNum = recruitPersonRelationService.dailyReportList(searchParams);

                GenerateReport generateReport = new GenerateReport();
                generateReport.setId(UUID.randomUUID().toString());
                generateReport.setCreateTime(new Date());
                generateReport.setDelFlag(false);
                generateReport.setDate(sdf.parse(date));
                generateReport.setLoginNum(loginNum);
                generateReport.setTotalLoginNum(totalLoginNum);
                generateReport.setDailyPersonRegNum(dailyPersonRegNum);
                generateReport.setTotalPersonRegNum(totalPersonRegNum);
                generateReport.setPersonNum(personNum);
                generateReport.setTotalPersonNum(totalPersonNum);
                generateReport.setTotalJobInformation(totalJobInformation);
                generateReport.setEnterpriseNum(enterpriseNum);
                generateReport.setTotalEnterpriseNum(totalEnterpriseNum);
                generateReport.setTotalRecruitStatusNum(totalRecruitStatusNum);
                generateReport.setTotalJobRequirementNum(totalJobRequirementNum);
                generateReport.setRecruitNum(recruitNum);
                generateReport.setJobRequirementNum(jobRequirementNum);
                generateReport.setTotalRecruitNum(totalRecruitNum);
                generateReport.setJobInformationNum(jobInformationNum);

                int count = generateReportService.insert(generateReport);
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
        }
    }

}
