package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.ShareWorksInfo;
import com.jpsoft.employment.modules.base.service.ShareWorksInfoService;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/shareWorksInfo")
public class ShareWorksInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ShareWorksInfoService shareWorksInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;
	
	
	@ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ShareWorksInfo> create(){
        MessageResult<ShareWorksInfo> msgResult = new MessageResult<>();

        ShareWorksInfo shareWorksInfo = new ShareWorksInfo();

        msgResult.setData(shareWorksInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ShareWorksInfo> add(@RequestBody ShareWorksInfo shareWorksInfo,@RequestAttribute String subject){
        MessageResult<ShareWorksInfo> msgResult = new MessageResult<>();

        try {
            shareWorksInfo.setId(UUID.randomUUID().toString());
			shareWorksInfo.setDelFlag(false);
            shareWorksInfo.setCreateBy(subject);
            shareWorksInfo.setCreateTime(new Date());
            shareWorksInfo.setStatus("0");

            int affectCount = shareWorksInfoService.insert(shareWorksInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(shareWorksInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ShareWorksInfo> edit(@PathVariable("id") String id){
        MessageResult<ShareWorksInfo> msgResult = new MessageResult<>();

        try {
            ShareWorksInfo shareWorksInfo = shareWorksInfoService.get(id);

            if (shareWorksInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(shareWorksInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ShareWorksInfo> update(@RequestBody ShareWorksInfo shareWorksInfo,@RequestAttribute String subject){
        MessageResult<ShareWorksInfo> msgResult = new MessageResult<>();

        try {
		    shareWorksInfo.setUpdateBy(subject);
            shareWorksInfo.setUpdateTime(new Date());
            shareWorksInfo.setStatus("0");
		
            int affectCount = shareWorksInfoService.update(shareWorksInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(shareWorksInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<ShareWorksInfo> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<ShareWorksInfo> msgResult = new MessageResult<>();

        try {
			
			ShareWorksInfo shareWorksInfo = shareWorksInfoService.get(id);
            shareWorksInfo.setDelFlag(true);
            shareWorksInfo.setUpdateBy(subject);
            shareWorksInfo.setUpdateTime(new Date());
			
			int affectCount = shareWorksInfoService.update(shareWorksInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                ShareWorksInfo shareWorksInfo = shareWorksInfoService.get(id);
                shareWorksInfo.setDelFlag(true);
                shareWorksInfo.setUpdateBy(subject);
                shareWorksInfo.setUpdateTime(new Date());

                affectCount += shareWorksInfoService.update(shareWorksInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String title,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(title)) {
            searchParams.put("title",title + "%");
        }


        Page<ShareWorksInfo> page = shareWorksInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);


        for (ShareWorksInfo shareWorksInfo:page) {
            String typeN = dataDictionaryService.findNameByCatalogNameAndValue("共享用工类型",shareWorksInfo.getType());
            if(StringUtils.isNotEmpty(typeN)){
                shareWorksInfo.setTypeN(typeN);
            }

        }


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="审核信息")
    @PostMapping("check/{id}")
    public MessageResult<ShareWorksInfo> check(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<ShareWorksInfo> msgResult = new MessageResult<>();

        try {

            ShareWorksInfo shareWorksInfo = shareWorksInfoService.get(id);
            shareWorksInfo.setStatus("1");
            shareWorksInfo.setUpdateBy(subject);
            shareWorksInfo.setUpdateTime(new Date());

            int affectCount = shareWorksInfoService.update(shareWorksInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库审核失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
