package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.base.entity.EnterpriseInfo;
import com.jpsoft.employment.modules.base.service.EnterpriseInfoService;
import com.jpsoft.employment.modules.common.dto.FileDTO;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.TrainingInfo;
import com.jpsoft.employment.modules.base.service.TrainingInfoService;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/trainingInfo")
public class TrainingInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private TrainingInfoService trainingInfoService;

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;


    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<TrainingInfo> create() {
        MessageResult<TrainingInfo> msgResult = new MessageResult<>();

        TrainingInfo trainingInfo = new TrainingInfo();

        msgResult.setData(trainingInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    public MessageResult<TrainingInfo> add(@RequestBody TrainingInfo trainingInfo, @RequestAttribute String subject) {
        MessageResult<TrainingInfo> msgResult = new MessageResult<>();

        try {
            trainingInfo.setId(UUID.randomUUID().toString());
            trainingInfo.setDelFlag(false);
            trainingInfo.setCreateBy(subject);
            trainingInfo.setCreateTime(new Date());
            trainingInfo.setStatus("0");

            if (trainingInfo.getFileList() != null && trainingInfo.getFileList().size() > 0) {
                List<FileDTO> fileList = trainingInfo.getFileList();

                JSONArray jsonArray = JSONArray.fromObject(fileList);

                trainingInfo.setFile(jsonArray.toString());
            }

            int affectCount = trainingInfoService.insert(trainingInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(trainingInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<TrainingInfo> edit(@PathVariable("id") String id) {
        MessageResult<TrainingInfo> msgResult = new MessageResult<>();

        try {
            TrainingInfo trainingInfo = trainingInfoService.get(id);

            if (trainingInfo != null) {
                if (StringUtils.isNotEmpty(trainingInfo.getFile())) {
                    JSONArray jsonArray = JSONArray.fromObject(trainingInfo.getFile());

                    List<FileDTO> fileDTOList = new ArrayList<>();

                    for (Object object : jsonArray) {
                        JSONObject jsonObject = JSONObject.fromObject(object);
                        String name = jsonObject.get("name").toString();
                        String url = jsonObject.get("url").toString();

                        FileDTO fileDTO = new FileDTO();
                        fileDTO.setName(name);
                        fileDTO.setUrl(url);
                        fileDTOList.add(fileDTO);
                    }

                    trainingInfo.setFileList(fileDTOList);
                }

                msgResult.setResult(true);
                msgResult.setData(trainingInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    public MessageResult<TrainingInfo> update(@RequestBody TrainingInfo trainingInfo, @RequestAttribute String subject) {
        MessageResult<TrainingInfo> msgResult = new MessageResult<>();

        try {
            trainingInfo.setUpdateBy(subject);
            trainingInfo.setUpdateTime(new Date());

            if (trainingInfo.getFileList() != null && trainingInfo.getFileList().size() > 0) {
                List<FileDTO> fileList = trainingInfo.getFileList();

                JSONArray jsonArray = JSONArray.fromObject(fileList);

                trainingInfo.setFile(jsonArray.toString());
            }

            int affectCount = trainingInfoService.update(trainingInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(trainingInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<TrainingInfo> delete(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<TrainingInfo> msgResult = new MessageResult<>();

        try {

            TrainingInfo trainingInfo = trainingInfoService.get(id);
            trainingInfo.setDelFlag(true);
            trainingInfo.setUpdateBy(subject);
            trainingInfo.setUpdateTime(new Date());

            int affectCount = trainingInfoService.update(trainingInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                TrainingInfo trainingInfo = trainingInfoService.get(id);
                trainingInfo.setDelFlag(true);
                trainingInfo.setUpdateBy(subject);
                trainingInfo.setUpdateTime(new Date());

                affectCount += trainingInfoService.update(trainingInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            HttpServletRequest request) {
        String subject = (String) request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String, Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time", "desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id", "%" + id + "%");
        }


        Page<TrainingInfo> page = trainingInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

        for (TrainingInfo trainingInfo : page) {
            EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(trainingInfo.getEnterpriseId());
            if (enterpriseInfo != null) {
                trainingInfo.setEnterpriseName(enterpriseInfo.getName());
            }


            if (StringUtils.isNotEmpty(trainingInfo.getFile())) {
                JSONArray jsonArray = JSONArray.fromObject(trainingInfo.getFile());

                List<FileDTO> fileDTOList = new ArrayList<>();

                for (Object object : jsonArray) {
                    JSONObject jsonObject = JSONObject.fromObject(object);
                    String name = jsonObject.get("name").toString();
                    String url = jsonObject.get("url").toString();

                    FileDTO fileDTO = new FileDTO();
                    fileDTO.setName(name);
                    fileDTO.setUrl(url);
                    fileDTOList.add(fileDTO);
                }

                trainingInfo.setFileList(fileDTOList);
            }
        }


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value = "审核信息")
    @PostMapping("check/{id}")
    public MessageResult<TrainingInfo> check(@PathVariable("id") String id, @RequestAttribute String subject) {
        MessageResult<TrainingInfo> msgResult = new MessageResult<>();

        try {

            TrainingInfo trainingInfo = trainingInfoService.get(id);
            trainingInfo.setStatus("1");
            trainingInfo.setUpdateBy(subject);
            trainingInfo.setUpdateTime(new Date());

            int affectCount = trainingInfoService.update(trainingInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库审核失败");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
