package com.jpsoft.employment.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.base.entity.PersonInfo;
import com.jpsoft.employment.modules.base.service.PersonInfoService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.base.entity.TrainingPerson;
import com.jpsoft.employment.modules.base.service.TrainingPersonService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/trainingPerson")
public class TrainingPersonController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private TrainingPersonService trainingPersonService;

    @Autowired
    private PersonInfoService personInfoService;
	
	
	@ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<TrainingPerson> create(){
        MessageResult<TrainingPerson> msgResult = new MessageResult<>();

        TrainingPerson trainingPerson = new TrainingPerson();

        msgResult.setData(trainingPerson);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<TrainingPerson> add(@RequestBody TrainingPerson trainingPerson,@RequestAttribute String subject){
        MessageResult<TrainingPerson> msgResult = new MessageResult<>();

        try {
            trainingPerson.setId(UUID.randomUUID().toString());
			trainingPerson.setDelFlag(false);
            trainingPerson.setCreateBy(subject);
            trainingPerson.setCreateTime(new Date());

            int affectCount = trainingPersonService.insert(trainingPerson);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(trainingPerson);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<TrainingPerson> edit(@PathVariable("id") String id){
        MessageResult<TrainingPerson> msgResult = new MessageResult<>();

        try {
            TrainingPerson trainingPerson = trainingPersonService.get(id);

            if (trainingPerson != null) {
                msgResult.setResult(true);
                msgResult.setData(trainingPerson);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<TrainingPerson> update(@RequestBody TrainingPerson trainingPerson,@RequestAttribute String subject){
        MessageResult<TrainingPerson> msgResult = new MessageResult<>();

        try {
		    trainingPerson.setUpdateBy(subject);
            trainingPerson.setUpdateTime(new Date());
		
            int affectCount = trainingPersonService.update(trainingPerson);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(trainingPerson);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<TrainingPerson> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<TrainingPerson> msgResult = new MessageResult<>();

        try {
			
			TrainingPerson trainingPerson = trainingPersonService.get(id);
            trainingPerson.setDelFlag(true);
            trainingPerson.setUpdateBy(subject);
            trainingPerson.setUpdateTime(new Date());
			
			int affectCount = trainingPersonService.update(trainingPerson);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                TrainingPerson trainingPerson = trainingPersonService.get(id);
                trainingPerson.setDelFlag(true);
                trainingPerson.setUpdateBy(subject);
                trainingPerson.setUpdateTime(new Date());

                affectCount += trainingPersonService.update(trainingPerson);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String trainingId,String name,String phone,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(trainingId)) {
            searchParams.put("trainingId",trainingId);
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name", name + "%");
        }

        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone", phone + "%");
        }


        Page<TrainingPerson> page = trainingPersonService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);


        for (TrainingPerson trainingPerson:page) {
            PersonInfo personInfo = personInfoService.get(trainingPerson.getPersonId());
            if(personInfo!=null){
                trainingPerson.setWorkPersonName(personInfo.getRealName());
                trainingPerson.setPhone(personInfo.getPhone());
                trainingPerson.setIdCard(personInfo.getIdCard());
                trainingPerson.setAge(personInfo.getAge());
                trainingPerson.setGender(personInfo.getGender());
            }
        }


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
