package com.jpsoft.employment.modules.mobile.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.jpsoft.employment.exception.CustomException;
import com.jpsoft.employment.modules.base.entity.PersonInfo;
import com.jpsoft.employment.modules.base.service.PersonInfoService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.JwtUtil;
import com.jpsoft.employment.modules.common.utils.SMSUtil;
import com.jpsoft.employment.modules.sys.entity.DataDictionary;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@RestController
@RequestMapping("/mobile/dictionary")
@Api(description = "字典接口")
public class DataDictionaryApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    RabbitTemplate rabbitTemplate;
    @Autowired
    private DataDictionaryService dataDictionaryService;

    @PostMapping("findListByCatalogName")
    @ApiOperation(value = " 根据字典名称查询字典(公开接口)")
    public MessageResult<Map> findListByCatalogName(String name) {
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
             List<DataDictionary> dataDictionaryList = dataDictionaryService.findByCatalogName(name);

            Map<String, Object> dataMap = new HashMap<String, Object>();
            dataMap.put("dictionaryList", dataDictionaryList);

            msgResult.setData(dataMap);
            msgResult.setResult(true);
        } catch (Exception ex) {
            if (ex instanceof CustomException) {
                log.error(ex.getMessage());
            } else {
                log.error(ex.getMessage(), ex);
            }

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @PostMapping("findValueByName")
    @ApiOperation(value = " 根据名称查内容")
    public MessageResult<String> findValueByName(String name) {
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            String value = dataDictionaryService.findValueByName(name);

            msgResult.setData(value);
            msgResult.setResult(true);
        } catch (Exception ex) {
            if (ex instanceof CustomException) {
                log.error(ex.getMessage());
            } else {
                log.error(ex.getMessage(), ex);
            }

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }
}
