package com.jpsoft.employment.modules.mobile.controller;

import com.alipay.api.domain.Person;
import com.github.pagehelper.Page;
import com.jpsoft.employment.modules.base.entity.*;
import com.jpsoft.employment.modules.base.service.*;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.common.utils.JwtUtil;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Slf4j
@RestController
@RequestMapping("/mobile/job")
@Api(description = "求职功能")
public class JobApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    RabbitTemplate rabbitTemplate;
    @Autowired
    private DataDictionaryService dataDictionaryService;
    @Autowired
    private ValueOperations<String, Object> valueOperations;
    @Autowired
    private PersonInfoService personInfoService;
    @Autowired
    private RecruitInformationInfoService recruitInformationInfoService;
    @Autowired
    private JobInformationInfoService jobInformationInfoService;
    @Autowired
    private JobEnterpriseRelationService jobEnterpriseRelationService;
    @Autowired
    private EnterpriseInfoService enterpriseInfoService;
    @Autowired
    private ShareWorksInfoService shareWorksInfoService;
    @Autowired
    private RecruitPersonRelationService recruitPersonRelationService;

    @ApiOperation(value="我的求职信息")
    @RequestMapping(value = "myJobHunt",method = RequestMethod.POST)
    public MessageResult<Map> myJobHunt(
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map returnMap = new HashMap();
            JobInformationInfo jobInformationInfo = jobInformationInfoService.findByPersonId(personInfo.getId());
            if(jobInformationInfo != null) {
                jobInformationInfo.setIntendedIndustriesN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业", jobInformationInfo.getIntendedIndustries()));
                jobInformationInfo.setMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式", jobInformationInfo.getMethod()));
                jobInformationInfo.setWorkExperienceN(dataDictionaryService.findNameByCatalogNameAndValue("工作经验", jobInformationInfo.getWorkExperience()));
                jobInformationInfo.setEducationN(dataDictionaryService.findNameByCatalogNameAndValue("学历", jobInformationInfo.getEducation()));
            }

            returnMap.put("jobInformationInfo",jobInformationInfo);
            msgResult.setData(returnMap);

            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="发布我的简历信")
    @RequestMapping(value = "createJobHunt",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "简历ID(传为修改", required = false, paramType = "form"),
            @ApiImplicitParam(name = "intendedIndustries", value = "意向行业", required = false, paramType = "form"),
            @ApiImplicitParam(name = "intendedPosition", value = "意向岗位", required = false, paramType = "form"),
            @ApiImplicitParam(name = "hopeSalary", value = "期望薪资(元/天", required = false, paramType = "form"),
            @ApiImplicitParam(name = "method", value = "结算方式", required = false, paramType = "form"),
            @ApiImplicitParam(name = "workExperience", value = "工作经验", required = false, paramType = "form"),
            @ApiImplicitParam(name = "education", value = "学历", required = false, paramType = "form"),
            @ApiImplicitParam(name = "serviceDesc", value = "服务描述", required = false, paramType = "form"),
            @ApiImplicitParam(name = "photo", value = "头像", required = false, paramType = "form"),
            @ApiImplicitParam(name = "address", value = "居住地", required = false, paramType = "form"),
    })
    public MessageResult<Map> createJobHunt(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="intendedIndustries",defaultValue="") String intendedIndustries,
            @RequestParam(value="intendedPosition",defaultValue="") String intendedPosition,
            @RequestParam(value="hopeSalary",defaultValue="") Integer hopeSalary,
            @RequestParam(value="method",defaultValue="") String method,
            @RequestParam(value="workExperience",defaultValue="") String workExperience,
            @RequestParam(value="education",defaultValue="") String education,
            @RequestParam(value="serviceDesc",defaultValue="") String serviceDesc,
            @RequestParam(value="photo",defaultValue="") String photo,
            @RequestParam(value="address",defaultValue="") String address,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            if(StringUtils.isNotEmpty(photo)) {
                //保存头像
                personInfo.setPhoto(photo);
                personInfoService.update(personInfo);
            }


            if (StringUtils.isNotEmpty(id)) {
                JobInformationInfo jobInformationInfo = jobInformationInfoService.get(id);
                jobInformationInfo.setUpdateBy(subject);
                jobInformationInfo.setUpdateTime(new Date());
                jobInformationInfo.setStatus("0");

                jobInformationInfo.setPersonId(subject);
                jobInformationInfo.setIntendedIndustries(intendedIndustries);
                jobInformationInfo.setIntendedPosition(intendedPosition);
                jobInformationInfo.setHopeSalary(hopeSalary);
                jobInformationInfo.setMethod(method);
                jobInformationInfo.setWorkExperience(workExperience);
                jobInformationInfo.setEducation(education);
                jobInformationInfo.setServiceDesc(serviceDesc);
                jobInformationInfo.setPhoto(photo);
                jobInformationInfo.setAddress(address);

                jobInformationInfoService.update(jobInformationInfo);
            } else {
                JobInformationInfo jobInformationInfo = jobInformationInfoService.findByPersonId(personInfo.getId());
                if(jobInformationInfo == null) {
                    jobInformationInfo = new JobInformationInfo();
                    jobInformationInfo.setId(UUID.randomUUID().toString());
                    jobInformationInfo.setDelFlag(false);
                    jobInformationInfo.setCreateBy(subject);
                    jobInformationInfo.setCreateTime(new Date());
                    jobInformationInfo.setUpdateTime(new Date());
                    jobInformationInfo.setStatus("0");

                    jobInformationInfo.setPersonId(subject);
                    jobInformationInfo.setIntendedIndustries(intendedIndustries);
                    jobInformationInfo.setIntendedPosition(intendedPosition);
                    jobInformationInfo.setHopeSalary(hopeSalary);
                    jobInformationInfo.setMethod(method);
                    jobInformationInfo.setWorkExperience(workExperience);
                    jobInformationInfo.setEducation(education);
                    jobInformationInfo.setServiceDesc(serviceDesc);
                    jobInformationInfo.setPhoto(photo);
                    jobInformationInfo.setAddress(address);

                    jobInformationInfoService.insert(jobInformationInfo);
                }else{
                    jobInformationInfo.setUpdateBy(subject);
                    jobInformationInfo.setUpdateTime(new Date());
                    jobInformationInfo.setStatus("0");

                    jobInformationInfo.setPersonId(subject);
                    jobInformationInfo.setIntendedIndustries(intendedIndustries);
                    jobInformationInfo.setIntendedPosition(intendedPosition);
                    jobInformationInfo.setHopeSalary(hopeSalary);
                    jobInformationInfo.setMethod(method);
                    jobInformationInfo.setWorkExperience(workExperience);
                    jobInformationInfo.setEducation(education);
                    jobInformationInfo.setServiceDesc(serviceDesc);
                    jobInformationInfo.setPhoto(photo);
                    jobInformationInfo.setAddress(address);

                    jobInformationInfoService.update(jobInformationInfo);
                }
            }

            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="求职信息详细(免登陆")
    @RequestMapping(value = "jobHuntDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "简历ID", required = false, paramType = "form"),
            @ApiImplicitParam(name = "isRead", value = "是否已读(1为已读", required = false, paramType = "form"),
            @ApiImplicitParam(name = "rprId", value = "关联表ID", required = false, paramType = "form"),
    })
    public MessageResult<Map> jobDetail(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="isRead",defaultValue="0") String isRead,
            @RequestParam(value="rprId",defaultValue="") String rprId,
            HttpServletRequest request){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {
            String subject = "";

            String token = request.getHeader("Authorization");

            if (org.springframework.util.StringUtils.isEmpty(token)){
                token = (String)request.getSession().getAttribute("token");
            }

            if (org.springframework.util.StringUtils.isEmpty(token)) {
                token = request.getParameter("token");
            }

            if (org.apache.commons.lang3.StringUtils.isNotEmpty(token)){
                subject = JwtUtil.decodeToken(jwtSecret,token);
            }

            Map<String,Object> returnMap = new HashMap<>();
            JobInformationInfo jobInformationInfo = jobInformationInfoService.get(id);
            PersonInfo personInfo = null;
            if(jobInformationInfo != null) {
                int BrowseNumber = 0;
                if(jobInformationInfo.getBrowseNumber() != null){
                    BrowseNumber = jobInformationInfo.getBrowseNumber();
                }
                BrowseNumber++;
                jobInformationInfo.setBrowseNumber(BrowseNumber);

                jobInformationInfoService.update(jobInformationInfo);

                personInfo = personInfoService.get(jobInformationInfo.getPersonId());

                String intendedIndustriesN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业",jobInformationInfo.getIntendedIndustries());
                if(StringUtils.isNotEmpty(intendedIndustriesN)){
                    jobInformationInfo.setIntendedIndustriesN(intendedIndustriesN);
                }

                String methodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式",jobInformationInfo.getMethod());
                if(StringUtils.isNotEmpty(methodN)){
                    jobInformationInfo.setMethodN(methodN);
                }

                String workExperienceN = dataDictionaryService.findNameByCatalogNameAndValue("工作经验",jobInformationInfo.getWorkExperience());
                if(StringUtils.isNotEmpty(workExperienceN)){
                    jobInformationInfo.setWorkExperienceN(workExperienceN);
                }


                String educationN = dataDictionaryService.findNameByCatalogNameAndValue("学历",jobInformationInfo.getEducation());
                if(StringUtils.isNotEmpty(educationN)){
                    jobInformationInfo.setEducationN(educationN);
                }
            }

            Boolean isInvite = false;
            //是否已邀请
            PersonInfo user = personInfoService.get(subject);
            if(user != null){
                JobEnterpriseRelation jobEnterpriseRelation = jobEnterpriseRelationService.findByEnterpriseIdAndJobInformationId(user.getEnterpriseId(),id);
                if(jobEnterpriseRelation != null) {
                    isInvite = true;
                }

                if("1".equals(isRead)) {
                    RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.get(rprId);
                    if(recruitPersonRelation != null) {
                        recruitPersonRelation.setIsRead(true);
                        recruitPersonRelation.setUpdateTime(new Date());
                        recruitPersonRelationService.update(recruitPersonRelation);
                    }
                }
            }


            returnMap.put("jobInformationInfo",jobInformationInfo);
            returnMap.put("personInfo",personInfo);
            returnMap.put("isInvite",isInvite);
            msgResult.setData(returnMap);
            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }


    @ApiOperation(value="我要报名(求职者报岗位")
    @RequestMapping(value = "joinRecruit",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "recruitId", value = "岗位ID", required = false, paramType = "form"),
    })
    public MessageResult<Map> joinRecruit(
            @RequestParam(value="recruitId",defaultValue="") String recruitId,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(recruitId);
            if(recruitInformationInfo == null){
                throw new Exception("未找到信息");
            }

            JobInformationInfo jobInformationInfo = jobInformationInfoService.findByPersonId(personInfo.getId());
            if(jobInformationInfo == null) {
                throw new Exception("未创建简历");
            }
            RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.findByRecruitIdAndPersonId(recruitId,personInfo.getId());
            if(recruitPersonRelation == null){
                recruitPersonRelation = new RecruitPersonRelation();
                recruitPersonRelation.setId(UUID.randomUUID().toString());
                recruitPersonRelation.setDelFlag(false);
                recruitPersonRelation.setCreateBy(subject);
                recruitPersonRelation.setCreateTime(new Date());
                recruitPersonRelation.setRecruitInformationId(recruitId);
                recruitPersonRelation.setWorkPersonId(subject);
                recruitPersonRelation.setJobInformationId(jobInformationInfo.getId());

                int count = recruitPersonRelationService.insert(recruitPersonRelation);

                if(count > 0){
                    msgResult.setResult(true);
                }else{
                    msgResult.setResult(false);
                }
            }
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }


    @ApiOperation(value="我的报名")
    @RequestMapping(value = "myRegistration",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "我的报名0,收到的邀请1,默认0", required = false, paramType = "form"),
    })
    public MessageResult<Map> myRegistration(
            @RequestParam(value="type",defaultValue="0") String type,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map<String,Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));

            if("0".equals(type)){
                searchParams.put("personId",subject);

                Page<RecruitPersonRelation> page = recruitPersonRelationService.pageSearchMobile(searchParams, pageIndex, pageSize, true, sortList);
                for (RecruitPersonRelation recruitPersonRelation : page.getResult()){
                    RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(recruitPersonRelation.getRecruitInformationId());
                    EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(recruitInformationInfo.getEnterpriseId());
                    if(enterpriseInfo!=null){
                        recruitInformationInfo.setEnterpriseName(enterpriseInfo.getName());
                    }

                    String industryN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry());
                    if(StringUtils.isNotEmpty(industryN)){
                        recruitInformationInfo.setIndustryN(industryN);
                    }

                    String settlementMethodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod());
                    if(StringUtils.isNotEmpty(settlementMethodN)){
                        recruitInformationInfo.setSettlementMethodN(settlementMethodN);
                    }

                    PersonInfo workPersonInfo = personInfoService.get(recruitPersonRelation.getWorkPersonId());
                    recruitPersonRelation.setWorkPersonInfo(workPersonInfo);
                    recruitPersonRelation.setRecruitInformationInfo(recruitInformationInfo);
                }

                msgResult.setResult(true);
                msgResult.setData(PojoUtils.pageWrapper(page));

            }else if("1".equals(type)){
                searchParams.put("personId",subject);

                Page<JobEnterpriseRelation> page = jobEnterpriseRelationService.pageSearchMobile(searchParams,pageIndex,pageSize,true,sortList);

                for (JobEnterpriseRelation jobEnterpriseRelation:page) {
                    RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(jobEnterpriseRelation.getRecruitInformationId());
                    EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(recruitInformationInfo.getEnterpriseId());
                    if(enterpriseInfo!=null){
                        recruitInformationInfo.setEnterpriseName(enterpriseInfo.getName());
                    }

                    String industryN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry());
                    if(StringUtils.isNotEmpty(industryN)){
                        recruitInformationInfo.setIndustryN(industryN);
                    }

                    String settlementMethodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod());
                    if(StringUtils.isNotEmpty(settlementMethodN)){
                        recruitInformationInfo.setSettlementMethodN(settlementMethodN);
                    }
                    jobEnterpriseRelation.setRecruitInformationInfo(recruitInformationInfo);
                }
                msgResult.setResult(true);
                msgResult.setData(PojoUtils.pageWrapper(page));
            }
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的报名(查未读数量")
    @RequestMapping(value = "myRegistrationNotReadNum",method = RequestMethod.POST)
    public MessageResult<Map> myRegistrationNotReadNum(
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map<String,Object> returnMap = new HashMap<>();
            //int rprNum = recruitPersonRelationService.findNotReadNumByPersonId(personInfo.getId());
            int jerNum = jobEnterpriseRelationService.findNotReadNumByPersonId(personInfo.getId());

            returnMap.put("notReadNum",jerNum);
            msgResult.setResult(true);
            msgResult.setData(returnMap);

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }
}
