package com.jpsoft.employment.modules.mobile.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.Page;
import com.jpsoft.employment.exception.CustomException;
import com.jpsoft.employment.modules.base.entity.*;
import com.jpsoft.employment.modules.base.service.*;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.dto.Sort;
import com.jpsoft.employment.modules.common.utils.JwtUtil;
import com.jpsoft.employment.modules.common.utils.PojoUtils;
import com.jpsoft.employment.modules.common.utils.SMSUtil;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import com.sun.org.apache.xpath.internal.operations.Bool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/mobile/recruit")
@Api(description = "招聘信息")
public class RecruitApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    RabbitTemplate rabbitTemplate;
    @Autowired
    private DataDictionaryService dataDictionaryService;
    @Autowired
    private ValueOperations<String, Object> valueOperations;
    @Autowired
    private PersonInfoService personInfoService;
    @Autowired
    private RecruitInformationInfoService recruitInformationInfoService;
    @Autowired
    private JobInformationInfoService jobInformationInfoService;
    @Autowired
    private JobEnterpriseRelationService jobEnterpriseRelationService;
    @Autowired
    private EnterpriseInfoService enterpriseInfoService;
    @Autowired
    private ShareWorksInfoService shareWorksInfoService;
    @Autowired
    private RecruitPersonRelationService recruitPersonRelationService;
    @Autowired
    private PersonLoginLogService personLoginLogService;


    @PostMapping("findRecruitHomePage")
    @ApiOperation(value = " 首页查询全部(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "(日结1,周结2,月结3 不传默认最新)", required = false, paramType = "form"),
    })
    public MessageResult<Map> findRecruitHomePage(
            @RequestParam(value="type",defaultValue="") String type,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            Map<String,Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.update_time","desc"));

            if (StringUtils.isNotEmpty(type)) {
                searchParams.put("settlementMethod",type);
            }

            searchParams.put("status","1");
            Page<RecruitInformationInfo> page = recruitInformationInfoService.pageSearchMobile(searchParams,pageIndex,pageSize,true,sortList);
            for(RecruitInformationInfo recruitInformationInfo : page.getResult()){
                recruitInformationInfo.setSettlementMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod()));
                recruitInformationInfo.setIndustryN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry()));
            }

            messageResult.setResult(true);
            messageResult.setData(PojoUtils.pageWrapper(page));
        } catch (Exception ex) {
            log.error(ex.getMessage());
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("findRecruitSearch")
    @ApiOperation(value = " 首搜索页面查询(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status", value = "查询是找工作0找零工1,不传默认找工作0)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "statusType", value = "找工作(日结1,周结2,月结3 不传默认最新)/找零工(最新1,最热2,不传默认最新)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "content", value = "搜索内容", required = false, paramType = "form"),
            @ApiImplicitParam(name = "industry", value = "行业(找工作)(找零工)字典表", required = false, paramType = "form"),
            @ApiImplicitParam(name = "settlementMethod", value = "结算方式(找工作)字典表", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salaryForm", value = "薪资形式(找工作)字典表", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salaryStart", value = "薪资待遇起找工作)(找零工)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salaryEnd", value = "薪资待遇止找工作)(找零工)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "workExp", value = "工作经验(字典表 1无,2半年,3一年,4两年,5三年,6五年,7十年及以上)(找零工)", required = false, paramType = "form"),
    })
    public MessageResult<Map> findRecruitSearch(
            @RequestParam(value="status",defaultValue="0") String status,
            @RequestParam(value="statusType",defaultValue="") String statusType,
            @RequestParam(value="content",defaultValue="") String content,
            @RequestParam(value="industry",defaultValue="") String industry,
            @RequestParam(value="settlementMethod",defaultValue="") String settlementMethod,
            @RequestParam(value="salaryForm",defaultValue="") String salaryForm,
            @RequestParam(value="salaryStart",defaultValue="") String salaryStart,
            @RequestParam(value="salaryEnd",defaultValue="") String salaryEnd,
            @RequestParam(value="workExp",defaultValue="") String workExp,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            Map<String,Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();

            if (StringUtils.isNotEmpty(industry)) {
                searchParams.put("industry",industry);
            }
            if (StringUtils.isNotEmpty(salaryStart)) {
                searchParams.put("salaryStart",salaryStart);
            }
            if (StringUtils.isNotEmpty(salaryEnd)) {
                searchParams.put("salaryEnd",salaryEnd);
            }

            if("0".equals(status)){
                //sortList.add(new Sort("IFNULL(a.is_topping,a.update_time)","desc"));
                sortList.add(new Sort("a.is_topping","desc"));
                sortList.add(new Sort("a.update_time","desc"));
                if (StringUtils.isNotEmpty(content)) {
                    searchParams.put("content","%" + content.trim() + "%");
                }

                if (StringUtils.isNotEmpty(statusType)) {
                    searchParams.put("settlementMethod",statusType);
                }

                if (StringUtils.isNotEmpty(settlementMethod)) {
                    searchParams.put("settlementMethod",settlementMethod);
                }
                if (StringUtils.isNotEmpty(salaryForm)) {
                    searchParams.put("salaryForm",salaryForm);
                }

                searchParams.put("status","1");
                searchParams.put("isOnline",true);
                Page<RecruitInformationInfo> page = recruitInformationInfoService.pageSearchMobile(searchParams,pageIndex,pageSize,true,sortList);
                for(RecruitInformationInfo recruitInformationInfo : page.getResult()){
                    recruitInformationInfo.setSettlementMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod()));
                    recruitInformationInfo.setIndustryN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry()));

                    PersonInfo personInfo = personInfoService.findByEnterpriseId(recruitInformationInfo.getEnterpriseId());
                    if(personInfo!=null){
                        PersonLoginLog personLoginLog = personLoginLogService.findTopByPersonId(personInfo.getId());
                        if(personLoginLog != null){
                            recruitInformationInfo.setLastLoginTime(personLoginLog.getCreateTime());
                        }
                    }
                }

                messageResult.setData(PojoUtils.pageWrapper(page));
            }else if("1".equals(status)){
                if (StringUtils.isNotEmpty(content)) {
                    searchParams.put("content","%" + content.trim() + "%");
                }

                if (StringUtils.isNotEmpty(workExp)) {
                    searchParams.put("workExp",workExp);
                }

                if (StringUtils.isNotEmpty(settlementMethod)) {
                    searchParams.put("method",settlementMethod);
                }

                if ("2".equals(statusType)) {
                    sortList.add(new Sort("a.browse_number","asc"));
                    sortList.add(new Sort("a.update_time","desc"));
                }else{
                    sortList.add(new Sort("a.update_time","desc"));
                }

                //searchParams.put("notIntention","2");
                searchParams.put("intention","1");//只显示公开的
                searchParams.put("jobStatus","1");//只显示公开的
                searchParams.put("status","1");
                Page<JobInformationInfo> page = jobInformationInfoService.pageSearchMobile(searchParams,pageIndex,pageSize,true,sortList);
                for (JobInformationInfo jobInformationInfo:page) {
                    PersonInfo personInfo = personInfoService.get(jobInformationInfo.getPersonId());
                    if(personInfo!=null){
                        jobInformationInfo.setPersonName(personInfo.getRealName());
                        jobInformationInfo.setGender(personInfo.getGender());
                        jobInformationInfo.setAge(personInfo.getAge());
                        PersonLoginLog personLoginLog = personLoginLogService.findTopByPersonId(personInfo.getId());
                        if(personLoginLog != null){
                            jobInformationInfo.setLastLoginTime(personLoginLog.getCreateTime());
                        }
                    }

                    jobInformationInfo.setIntendedIndustriesN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业",jobInformationInfo.getIntendedIndustries()));
                    jobInformationInfo.setMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式",jobInformationInfo.getMethod()));
                    jobInformationInfo.setWorkExperienceN(dataDictionaryService.findNameByCatalogNameAndValue("工作经验",jobInformationInfo.getWorkExperience()));
                    jobInformationInfo.setEducationN(dataDictionaryService.findNameByCatalogNameAndValue("学历",jobInformationInfo.getEducation()));

                }

                messageResult.setData(PojoUtils.pageWrapper(page));
            }

            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage());
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("findJobList")
    @ApiOperation(value = " 弃用-零工驿站(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "(日结1,周结2,月结3 不传默认最新)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salaryStart", value = "薪资待遇起(找零工)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salaryEnd", value = "薪资待遇止(找零工)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "workExp", value = "工作经验(字典表 1无,2半年,3一年,4两年,5三年,6五年,7十年及以上)(找零工)", required = false, paramType = "form"),
    })
    public MessageResult<Map> findJobList(
            @RequestParam(value="type",defaultValue="") String type,
            @RequestParam(value="salaryStart",defaultValue="") String salaryStart,
            @RequestParam(value="salaryEnd",defaultValue="") String salaryEnd,
            @RequestParam(value="workExp",defaultValue="") String workExp,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            Map<String,Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.update_time","desc"));

            if (StringUtils.isNotEmpty(type)) {
                searchParams.put("settlementMethod",type);
            }

            if (StringUtils.isNotEmpty(salaryStart)) {
                searchParams.put("salaryStart",salaryStart);
            }
            if (StringUtils.isNotEmpty(salaryEnd)) {
                searchParams.put("salaryEnd",salaryEnd);
            }

            if (StringUtils.isNotEmpty(workExp)) {
                searchParams.put("workExp",workExp);
            }

            searchParams.put("status","1");
            Page<RecruitInformationInfo> page = recruitInformationInfoService.pageSearchMobile(searchParams,pageIndex,pageSize,true,sortList);
            for(RecruitInformationInfo recruitInformationInfo : page.getResult()){
                recruitInformationInfo.setSettlementMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod()));
                recruitInformationInfo.setIndustryN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry()));
            }

            messageResult.setResult(true);
            messageResult.setData(PojoUtils.pageWrapper(page));
        } catch (Exception ex) {
            log.error(ex.getMessage());
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="发布招聘信息")
    @RequestMapping(value = "createRecruit",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID(传为修改", required = false, paramType = "form"),
            @ApiImplicitParam(name = "positionName", value = "职位名称", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salary", value = "薪资待遇", required = false, paramType = "form"),
            @ApiImplicitParam(name = "settlementMethod", value = "结算方式", required = false, paramType = "form"),
            @ApiImplicitParam(name = "industry", value = "行业", required = false, paramType = "form"),
            @ApiImplicitParam(name = "recruitingNumbers", value = "招聘人数", required = false, paramType = "form"),
            @ApiImplicitParam(name = "desc", value = "工作描述", required = false, paramType = "form"),
            @ApiImplicitParam(name = "contacts", value = "联系人", required = false, paramType = "form"),
            @ApiImplicitParam(name = "contactsPhone", value = "联系电话", required = false, paramType = "form"),
            @ApiImplicitParam(name = "workArea", value = "工作地区", required = false, paramType = "form"),
            @ApiImplicitParam(name = "address", value = "详细地址", required = false, paramType = "form"),
            @ApiImplicitParam(name = "salaryForm", value = "薪资形式(字典:计件,计时,定额)", required = false, paramType = "form"),
            @ApiImplicitParam(name = "isOnline", value = "是否上架", required = false, paramType = "form"),
            @ApiImplicitParam(name = "ageRequirement", value = "年龄要求", required = false, paramType = "form"),
    })
    public MessageResult<Map> createRecruit(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="positionName",defaultValue="") String positionName,
            @RequestParam(value="salary",defaultValue="") String salary,
            @RequestParam(value="settlementMethod",defaultValue="") String settlementMethod,
            @RequestParam(value="industry",defaultValue="") String industry,
            @RequestParam(value="recruitingNumbers",defaultValue="") String recruitingNumbers,
            @RequestParam(value="desc",defaultValue="") String desc,
            @RequestParam(value="contacts",defaultValue="") String contacts,
            @RequestParam(value="contactsPhone",defaultValue="") String contactsPhone,
            @RequestParam(value="workArea",defaultValue="") String workArea,
            @RequestParam(value="address",defaultValue="") String address,
            @RequestParam(value="salaryForm",defaultValue="") String salaryForm,
            @RequestParam(value="isOnline",defaultValue="true") Boolean isOnline,
            @RequestParam(value="ageRequirement",defaultValue="") String ageRequirement,
            @RequestParam(value="images",defaultValue="") String images,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            if(StringUtils.isEmpty(personInfo.getEnterpriseId())){
                throw new Exception("未绑定企业");
            }

            if (StringUtils.isNotEmpty(id)) {
                RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
                recruitInformationInfo.setUpdateBy(subject);
                recruitInformationInfo.setUpdateTime(new Date());
                recruitInformationInfo.setStatus("0");
                recruitInformationInfo.setEnterpriseId(personInfo.getEnterpriseId());

                recruitInformationInfo.setPositionName(positionName);
                recruitInformationInfo.setSalary(salary);
                recruitInformationInfo.setSettlementMethod(settlementMethod);
                recruitInformationInfo.setIndustry(industry);
                recruitInformationInfo.setRecruitingNumbers(recruitingNumbers);
                recruitInformationInfo.setDesc(desc);
                recruitInformationInfo.setContacts(contacts);
                recruitInformationInfo.setContactsPhone(contactsPhone);
                recruitInformationInfo.setWorkArea(workArea);
                recruitInformationInfo.setAddress(address);
                recruitInformationInfo.setSalaryForm(salaryForm);
                recruitInformationInfo.setIsOnline(isOnline);
                recruitInformationInfo.setImages(images);
                recruitInformationInfo.setAgeRequirement(ageRequirement);

                recruitInformationInfoService.update(recruitInformationInfo);
            } else {
                RecruitInformationInfo recruitInformationInfo = new RecruitInformationInfo();
                recruitInformationInfo.setId(UUID.randomUUID().toString());
                recruitInformationInfo.setDelFlag(false);
                recruitInformationInfo.setCreateBy(subject);
                recruitInformationInfo.setCreateTime(new Date());
                recruitInformationInfo.setUpdateTime(new Date());
                recruitInformationInfo.setStatus("0");
                recruitInformationInfo.setEnterpriseId(personInfo.getEnterpriseId());

                recruitInformationInfo.setPositionName(positionName);
                recruitInformationInfo.setSalary(salary);
                recruitInformationInfo.setSettlementMethod(settlementMethod);
                recruitInformationInfo.setIndustry(industry);
                recruitInformationInfo.setRecruitingNumbers(recruitingNumbers);
                recruitInformationInfo.setDesc(desc);
                recruitInformationInfo.setContacts(contacts);
                recruitInformationInfo.setContactsPhone(contactsPhone);
                recruitInformationInfo.setWorkArea(workArea);
                recruitInformationInfo.setAddress(address);
                recruitInformationInfo.setSalaryForm(salaryForm);
                recruitInformationInfo.setIsOnline(isOnline);
                recruitInformationInfo.setAgeRequirement(ageRequirement);
                recruitInformationInfo.setImages(images);
                recruitInformationInfo.setIsTopping(false);

                recruitInformationInfoService.insert(recruitInformationInfo);
            }

            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="修改招聘信息")
    @RequestMapping(value = "updateRecruitOnline",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = false, paramType = "form"),
            @ApiImplicitParam(name = "isOnline", value = "上架状态", required = false, paramType = "form"),
    })
    public MessageResult<Map> updateRecruitOnline(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="isOnline",defaultValue="true") Boolean isOnline,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            if(recruitInformationInfo == null){
                throw new Exception("未找到岗位");
            }
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());
            recruitInformationInfo.setIsOnline(isOnline);
            recruitInformationInfoService.update(recruitInformationInfo);


            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="招聘信息详细(公开接口")
    @RequestMapping(value = "recruitDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "recruitId", value = "招聘信息ID", required = false, paramType = "form"),
            @ApiImplicitParam(name = "isRead", value = "是否已读(1为已读", required = false, paramType = "form"),
            @ApiImplicitParam(name = "jerId", value = "关联表ID", required = false, paramType = "form"),
    })
    public MessageResult<Map> recruitDetail(
            @RequestParam(value="recruitId",defaultValue="") String recruitId,
            @RequestParam(value="isRead",defaultValue="0") String isRead,
            @RequestParam(value="jerId",defaultValue="") String jerId,
            HttpServletRequest request){
        MessageResult<Map> msgResult = new MessageResult<>();
        try {
            String subject = "";

            String token = request.getHeader("Authorization");

            if (org.springframework.util.StringUtils.isEmpty(token)){
                token = (String)request.getSession().getAttribute("token");
            }

            if (org.springframework.util.StringUtils.isEmpty(token)) {
                token = request.getParameter("token");
            }

            if (org.apache.commons.lang3.StringUtils.isNotEmpty(token)){
                subject = JwtUtil.decodeToken(jwtSecret,token);
            }

            Boolean isJoin = false;

            Map<String,Object> returnMap = new HashMap<>();
            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(recruitId);
            if(recruitInformationInfo == null){
                throw new Exception("未找到信息");
            }

            int BrowseNumber = 0;
            if(recruitInformationInfo.getBrowseNumber() != null){
                BrowseNumber = recruitInformationInfo.getBrowseNumber();
            }
            BrowseNumber++;
            recruitInformationInfo.setBrowseNumber(BrowseNumber);
            recruitInformationInfoService.update(recruitInformationInfo);


            EnterpriseInfo enterpriseInfo = null;
            if(recruitInformationInfo.getEnterpriseId() != null) {
                enterpriseInfo = enterpriseInfoService.get(recruitInformationInfo.getEnterpriseId());
            }

            PersonInfo personInfo = personInfoService.get(subject);
            if(personInfo != null){
                RecruitPersonRelation recruitPersonRelation = recruitPersonRelationService.findByRecruitIdAndPersonId(recruitId,personInfo.getId());
                if(recruitPersonRelation != null){
                    isJoin = true;
                }

                if("1".equals(isRead)) {
                    JobEnterpriseRelation jobEnterpriseRelation = jobEnterpriseRelationService.get(jerId);
                    if(jobEnterpriseRelation != null) {
                        jobEnterpriseRelation.setIsRead(true);
                        jobEnterpriseRelation.setUpdateTime(new Date());
                        jobEnterpriseRelationService.update(jobEnterpriseRelation);
                    }
                }
            }

            recruitInformationInfo.setIndustryN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry()));
            recruitInformationInfo.setSettlementMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod()));
            recruitInformationInfo.setSalaryFormN(dataDictionaryService.findNameByCatalogNameAndValue("薪资形式",recruitInformationInfo.getSalaryForm()));



            returnMap.put("recruitInformationInfo",recruitInformationInfo);
            returnMap.put("enterpriseInfo",enterpriseInfo);
            returnMap.put("isJoin",isJoin);

            msgResult.setData(returnMap);
            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="发布共享用工信息")
    @RequestMapping(value = "createShareWork",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID(传为修改", required = false, paramType = "form"),
            @ApiImplicitParam(name = "type", value = "信息类型", required = false, paramType = "form"),
            @ApiImplicitParam(name = "title", value = "标题", required = false, paramType = "form"),
            @ApiImplicitParam(name = "content", value = "内容", required = false, paramType = "form"),
    })
    public MessageResult<Map> createShareWork(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="type",defaultValue="") String type,
            @RequestParam(value="title",defaultValue="") String title,
            @RequestParam(value="content",defaultValue="") String content,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            if(StringUtils.isEmpty(personInfo.getEnterpriseId())){
                throw new Exception("未绑定企业");
            }

            if (StringUtils.isNotEmpty(id)) {
                ShareWorksInfo shareWorksInfo = shareWorksInfoService.get(id);
                shareWorksInfo.setUpdateBy(subject);
                shareWorksInfo.setUpdateTime(new Date());
                shareWorksInfo.setStatus("0");

                shareWorksInfo.setType(type);
                shareWorksInfo.setTitle(title);
                shareWorksInfo.setContent(content);
                shareWorksInfo.setAuthor(personInfo.getRealName());

                shareWorksInfoService.update(shareWorksInfo);
            } else {
                ShareWorksInfo shareWorksInfo = new ShareWorksInfo();
                shareWorksInfo.setId(UUID.randomUUID().toString());
                shareWorksInfo.setDelFlag(false);
                shareWorksInfo.setCreateBy(subject);
                shareWorksInfo.setCreateTime(new Date());
                shareWorksInfo.setStatus("0");

                shareWorksInfo.setType(type);
                shareWorksInfo.setTitle(title);
                shareWorksInfo.setContent(content);
                shareWorksInfo.setAuthor(personInfo.getRealName());

                shareWorksInfoService.insert(shareWorksInfo);
            }

            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的用工管理")
    @RequestMapping(value = "myEmployment",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status", value = "发布中1,待审核0,未通过2", required = false, paramType = "form"),
            @ApiImplicitParam(name = "isOnline", value = "是否上架", required = false, paramType = "form"),
    })
    public MessageResult<Map> myEmployment(
            @RequestParam(value="status",defaultValue="1") String status,
            @RequestParam(value="isOnline",defaultValue="") Boolean isOnline,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map<String,Object> searchParams = new HashMap<>();

            if(StringUtils.isEmpty(personInfo.getEnterpriseId())){
                personInfo.setEnterpriseId("wufachakan");
            }
            searchParams.put("enterpriseId",personInfo.getEnterpriseId());

            searchParams.put("status",status);
            if(isOnline != null) {
                searchParams.put("isOnline", isOnline);
            }

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time","desc"));

            Page<RecruitInformationInfo> page = recruitInformationInfoService.pageSearchMobile(searchParams, pageIndex, pageSize, true, sortList);

            msgResult.setResult(true);
            msgResult.setData(PojoUtils.pageWrapper(page));

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的用工管理刷新")
    @RequestMapping(value = "myEmploymentRefresh",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", required = false, paramType = "form"),
    })
    public MessageResult<Boolean> myEmploymentRefresh(
            @RequestParam(value="id",defaultValue="1") String id,
            @RequestAttribute String subject){

        MessageResult<Boolean> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            recruitInformationInfo.setUpdateBy(subject);
            recruitInformationInfo.setUpdateTime(new Date());
            int count = recruitInformationInfoService.update(recruitInformationInfo);
            Boolean returnFlag = false;
            if(count > 0){
                returnFlag = true;
            }

            msgResult.setResult(true);
            msgResult.setData(returnFlag);

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的用工管理删除")
    @RequestMapping(value = "myEmploymentDelete",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", required = false, paramType = "form"),
    })
    public MessageResult<Boolean> myEmploymentDelete(
            @RequestParam(value="id",defaultValue="1") String id,
            @RequestAttribute String subject){

        MessageResult<Boolean> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Boolean returnFlag = false;
            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(id);
            if(recruitInformationInfo != null) {
                recruitInformationInfo.setDelFlag(true);
                recruitInformationInfo.setUpdateBy(subject);
                recruitInformationInfo.setUpdateTime(new Date());
                int count = recruitInformationInfoService.update(recruitInformationInfo);
                if (count > 0) {
                    returnFlag = true;
                }
            }

            msgResult.setResult(true);
            msgResult.setData(returnFlag);

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的收到报名")
    @RequestMapping(value = "myReceivedRegistration",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status", value = "收到报名0,邀请记录1", required = false, paramType = "form"),
    })
    public MessageResult<Map> myReceivedRegistration(
            @RequestParam(value="status",defaultValue="0") String status,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map<String,Object> searchParams = new HashMap<>();


            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time","desc"));
            if("0".equals(status)){
                searchParams.put("enterpriseId",personInfo.getEnterpriseId());

                Page<RecruitPersonRelation> page = recruitPersonRelationService.pageSearchMobile(searchParams, pageIndex, pageSize, true, sortList);
                for(RecruitPersonRelation recruitPersonRelation : page.getResult()){
                    RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(recruitPersonRelation.getRecruitInformationId());

                    EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(recruitInformationInfo.getEnterpriseId());
                    if(enterpriseInfo!=null){
                        recruitInformationInfo.setEnterpriseName(enterpriseInfo.getName());
                    }

                    recruitInformationInfo.setIndustryN(dataDictionaryService.findNameByCatalogNameAndValue("意向行业",recruitInformationInfo.getIndustry()));
                    recruitInformationInfo.setSettlementMethodN(dataDictionaryService.findNameByCatalogNameAndValue("结算方式",recruitInformationInfo.getSettlementMethod()));

                    PersonInfo workPersonInfo = personInfoService.get(recruitPersonRelation.getWorkPersonId());
                    recruitPersonRelation.setWorkPersonInfo(workPersonInfo);

                    JobInformationInfo jobInformationInfo = jobInformationInfoService.get(recruitPersonRelation.getJobInformationId());
                    String intendedIndustriesN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业",jobInformationInfo.getIntendedIndustries());
                    if(StringUtils.isNotEmpty(intendedIndustriesN)){
                        jobInformationInfo.setIntendedIndustriesN(intendedIndustriesN);
                    }

                    String methodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式",jobInformationInfo.getMethod());
                    if(StringUtils.isNotEmpty(methodN)){
                        jobInformationInfo.setMethodN(methodN);
                    }

                    String workExperienceN = dataDictionaryService.findNameByCatalogNameAndValue("工作经验",jobInformationInfo.getWorkExperience());
                    if(StringUtils.isNotEmpty(workExperienceN)){
                        jobInformationInfo.setWorkExperienceN(workExperienceN);
                    }


                    String educationN = dataDictionaryService.findNameByCatalogNameAndValue("学历",jobInformationInfo.getEducation());
                    if(StringUtils.isNotEmpty(educationN)){
                        jobInformationInfo.setEducationN(educationN);
                    }

                    recruitPersonRelation.setJobInformationInfo(jobInformationInfo);

                    recruitPersonRelation.setRecruitInformationInfo(recruitInformationInfo);
                }
                msgResult.setResult(true);
                msgResult.setData(PojoUtils.pageWrapper(page));

            }else if("1".equals(status)){
                searchParams.put("enterpriseId",personInfo.getEnterpriseId());

                Page<JobEnterpriseRelation> page = jobEnterpriseRelationService.pageSearchMobile(searchParams, pageIndex, pageSize, true, sortList);
                for(JobEnterpriseRelation jobEnterpriseRelation : page.getResult()){
                    JobInformationInfo jobInformationInfo = jobInformationInfoService.get(jobEnterpriseRelation.getJobInformationId());
                    String intendedIndustriesN = dataDictionaryService.findNameByCatalogNameAndValue("意向行业",jobInformationInfo.getIntendedIndustries());
                    if(StringUtils.isNotEmpty(intendedIndustriesN)){
                        jobInformationInfo.setIntendedIndustriesN(intendedIndustriesN);
                    }

                    String methodN = dataDictionaryService.findNameByCatalogNameAndValue("结算方式",jobInformationInfo.getMethod());
                    if(StringUtils.isNotEmpty(methodN)){
                        jobInformationInfo.setMethodN(methodN);
                    }

                    String workExperienceN = dataDictionaryService.findNameByCatalogNameAndValue("工作经验",jobInformationInfo.getWorkExperience());
                    if(StringUtils.isNotEmpty(workExperienceN)){
                        jobInformationInfo.setWorkExperienceN(workExperienceN);
                    }


                    String educationN = dataDictionaryService.findNameByCatalogNameAndValue("学历",jobInformationInfo.getEducation());
                    if(StringUtils.isNotEmpty(educationN)){
                        jobInformationInfo.setEducationN(educationN);
                    }

                    jobEnterpriseRelation.setJobInformationInfo(jobInformationInfo);

                    PersonInfo workPersonInfo = personInfoService.get(jobInformationInfo.getPersonId());
                    jobEnterpriseRelation.setWorkPersonInfo(workPersonInfo);

                    RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(jobEnterpriseRelation.getRecruitInformationId());
                    jobEnterpriseRelation.setRecruitInformationInfo(recruitInformationInfo);
                }
                msgResult.setResult(true);
                msgResult.setData(PojoUtils.pageWrapper(page));
            }

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的收到报名(查未读数量")
    @RequestMapping(value = "myReceivedRegistrationNotReadNum",method = RequestMethod.POST)
    public MessageResult<Map> myReceivedRegistrationNotReadNum(
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map<String,Object> returnMap = new HashMap<>();
            int rprNum = recruitPersonRelationService.findNotReadNumByEnterpriseId(personInfo.getEnterpriseId());
            //int jerNum = jobEnterpriseRelationService.findNotReadNumByEnterpriseId(personInfo.getEnterpriseId());

            returnMap.put("notReadNum",rprNum);
            msgResult.setResult(true);
            msgResult.setData(returnMap);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }


    @ApiOperation(value="我的共享用工")
    @RequestMapping(value = "myShareWork",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status", value = "发布中1,待审核0,未通过2", required = false, paramType = "form"),
    })
    public MessageResult<Map> myShareWork(
            @RequestParam(value="status",defaultValue="1") String status,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            Map<String,Object> searchParams = new HashMap<>();


            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time","desc"));

            searchParams.put("createBy",subject);
            searchParams.put("status",status);

            Page<ShareWorksInfo> page = shareWorksInfoService.pageSearchMobile(searchParams, pageIndex, pageSize, true, sortList);

            msgResult.setResult(true);
            msgResult.setData(PojoUtils.pageWrapper(page));

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="我的共享用工删除")
    @RequestMapping(value = "myShareWorkDelete",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", required = false, paramType = "form"),
    })
    public MessageResult<Boolean> myShareWorkDelete(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestAttribute String subject){

        MessageResult<Boolean> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }
            Boolean returnFlag = false;
            ShareWorksInfo shareWorksInfo = shareWorksInfoService.get(id);
            if(shareWorksInfo != null){
                shareWorksInfo.setDelFlag(true);
                shareWorksInfo.setUpdateBy(subject);
                shareWorksInfo.setUpdateTime(new Date());
                int count = shareWorksInfoService.update(shareWorksInfo);
                if(count > 0 ){
                    returnFlag = true;
                }
            }

            msgResult.setResult(true);
            msgResult.setData(returnFlag);

        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="邀请工人(企业邀请简历")
    @RequestMapping(value = "inviteJob",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "jobId", value = "简历ID", required = false, paramType = "form"),
            @ApiImplicitParam(name = "recruitId", value = "岗位ID", required = false, paramType = "form"),
    })
    public MessageResult<Map> inviteJob(
            @RequestParam(value="jobId",defaultValue="") String jobId,
            @RequestParam(value="recruitId",defaultValue="") String recruitId,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            RecruitInformationInfo recruitInformationInfo = recruitInformationInfoService.get(recruitId);
            if(recruitInformationInfo == null){
                throw new Exception("未找到岗位信息");
            }

            JobInformationInfo jobInformationInfo = jobInformationInfoService.get(jobId);
            if(jobInformationInfo == null){
                throw new Exception("未找到简历信息");
            }

            JobEnterpriseRelation jobEnterpriseRelation = jobEnterpriseRelationService.findByEnterpriseIdAndJobInformationId(recruitInformationInfo.getEnterpriseId(),jobId);
            if(jobEnterpriseRelation == null){
                jobEnterpriseRelation = new JobEnterpriseRelation();
                jobEnterpriseRelation.setId(UUID.randomUUID().toString());
                jobEnterpriseRelation.setCreateBy(subject);
                jobEnterpriseRelation.setCreateTime(new Date());
                jobEnterpriseRelation.setDelFlag(false);

                jobEnterpriseRelation.setEnterpriseId(recruitInformationInfo.getEnterpriseId());
                jobEnterpriseRelation.setJobInformationId(jobId);
                jobEnterpriseRelation.setRecruitInformationId(recruitId);
                int count = jobEnterpriseRelationService.insert(jobEnterpriseRelation);
                if(count > 0){
                    msgResult.setResult(true);
                }else{
                    msgResult.setResult(false);
                }

            }else{
                throw new Exception("已经邀请");
            }
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }
}
