package com.jpsoft.employment.modules.mobile.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.jpsoft.employment.exception.CustomException;
import com.jpsoft.employment.modules.base.entity.*;
import com.jpsoft.employment.modules.base.service.EnterpriseInfoService;
import com.jpsoft.employment.modules.base.service.JobInformationInfoService;
import com.jpsoft.employment.modules.base.service.PersonInfoService;
import com.jpsoft.employment.modules.base.service.PersonLoginLogService;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.DES3;
import com.jpsoft.employment.modules.common.utils.JwtUtil;
import com.jpsoft.employment.modules.common.utils.SMSUtil;
import com.jpsoft.employment.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Slf4j
@RestController
@RequestMapping("/mobile/user")
@Api(description = "用户接口")
public class UserApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    RabbitTemplate rabbitTemplate;
    @Autowired
    private DataDictionaryService dataDictionaryService;
    @Autowired
    private ValueOperations<String, Object> valueOperations;
    @Autowired
    private PersonInfoService personInfoService;
    @Autowired
    private EnterpriseInfoService enterpriseInfoService;
    @Autowired
    private JobInformationInfoService jobInformationInfoService;
    @Autowired
    private PersonLoginLogService personLoginLogService;


    @GetMapping("findByOpenId")
    public MessageResult<Map> findByOpenId(String openId) {
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            PersonInfo regUser= personInfoService.findByOpenId(openId);

            if (regUser != null) {
                String token = JwtUtil.createToken(jwtSecret, regUser.getId(), DateTime.now().plusDays(7).toDate());

                EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(regUser.getEnterpriseId());

                JobInformationInfo jobInformationInfo = jobInformationInfoService.findByPersonId(regUser.getId());

                Map<String, Object> dataMap = new HashMap<String, Object>();
                dataMap.put("regUser", regUser);
                dataMap.put("enterpriseInfo", enterpriseInfo);
                dataMap.put("jobInformationInfo", jobInformationInfo);
                dataMap.put("token", token);

                msgResult.setData(dataMap);
                msgResult.setResult(true);
            } else {
                throw new CustomException("openid不存在");
            }
        } catch (Exception ex) {
            if (ex instanceof CustomException) {
                log.error(ex.getMessage());
            } else {
                log.error(ex.getMessage(), ex);
            }

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @GetMapping("logout")
    public MessageResult<Map> logout(@RequestAttribute String subject) {
        MessageResult<Map> msgResult = new MessageResult<>();
        try {
            int count = personInfoService.clearOpenId(subject);
            if (count > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
            }
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @GetMapping("deleteUser")
    public MessageResult<String> deleteUser(@RequestAttribute String subject) {
        MessageResult<String> msgResult = new MessageResult<>();

        try{
            PersonInfo personInfo = personInfoService.get(subject);
            if(personInfo == null){
                throw new Exception("未找到申请用户!");
            }

            personInfo.setDelFlag(true);
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());
            int count = personInfoService.update(personInfo);
            if(count > 0) {
                msgResult.setResult(true);
            }else{
                msgResult.setData("0");
                msgResult.setResult(false);
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @PostMapping("getVerifyCode")
    @ApiOperation(value = "获取短信验证码(公开接口)")
    public MessageResult<Map> getVerifyCode(String telephone) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            String key = "SMS_" + telephone;

            //1分钟限制
            boolean repeat = valueOperations.setIfAbsent("out_" + telephone,true,1, TimeUnit.MINUTES);
            if(!repeat){
                throw new Exception("操作过于频繁!");
            }

            String verifyCode = (String) valueOperations.get(key);

            if (StringUtils.isEmpty(verifyCode)) {
                verifyCode = SMSUtil.generateNumberString(6);
                JSONObject verifyCodeJSON = new JSONObject();
                verifyCodeJSON.put("code", verifyCode);

                messageResult = SMSUtil.send(telephone, "SMS_49390047", verifyCodeJSON.toString());

                //设置当前用户的验证码,5分钟内有效
                valueOperations.set(key, verifyCode, 10, TimeUnit.MINUTES);
            }

            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage());
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("validateCode")
    @ApiOperation(value = "验证短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "headImg", value = "头像", required = false, paramType = "form"),
    })
    public MessageResult<Map> validateCode(String verifyCode,
                                           String openId,
                                           String nickName,
                                           String telephone,
                                           String headImg,
                                           @RequestParam(value="source",defaultValue="2") String source) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            String smsKey = "SMS_" + telephone;

            String beforeVerifyCode = (String) valueOperations.get(smsKey);
            if(!"9527".equals(verifyCode)) {
                if (StringUtils.isEmpty(beforeVerifyCode)) {
                    throw new Exception("验证码已过期!");
                }

                if (!beforeVerifyCode.equals(verifyCode)) {
                    throw new Exception("验证码错误!");
                }
            }

            if (StrUtil.isBlank(openId)) {
                throw new Exception("微信标识不存在");
            }

            PersonInfo personInfo = null;
            String token = null;
            if (StringUtils.isNotEmpty(openId)) {
                personInfo = personInfoService.findByPhone(telephone);

                if (personInfo != null) {
                    personInfo.setOpenId(openId);
                    personInfo.setUpdateTime(new Date());
                    personInfoService.update(personInfo);
                    token = JwtUtil.createToken(jwtSecret, personInfo.getId(), DateTime.now().plusHours(6).toDate());


                } else {
                    throw new Exception("未找到用户,请联系管理员!");
                }
            }

            Map<String, Object> map = new HashMap<>();
            map.put("token", token);
            map.put("regUser", personInfo);

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            ex.printStackTrace();
            log.error(ex.getMessage());
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("changePhone")
    public MessageResult<Map> changePhone(
            String verifyCode, String telephone,
            @RequestAttribute String subject) {
        MessageResult<Map> msgResult = new MessageResult<>();

        try{
//            String smsKey = "SMS_" + telephone;

//            String beforeVerifyCode = (String) valueOperations.get(smsKey);
//
//            if (StringUtils.isEmpty(beforeVerifyCode)) {
//                throw new Exception("验证码已过期!");
//            }
//
//            if (!beforeVerifyCode.equals(verifyCode)) {
//                throw new Exception("验证码错误!");
//            }

            PersonInfo oldUser = personInfoService.findByPhone(telephone);
            if(oldUser != null){
                if(telephone.equals(oldUser.getPhone())){
                    //手机号相同
                    throw new Exception("手机号与原号码相同");
                }else {
                    throw new Exception("手机号已被注册");
                }
            }

            PersonInfo user = personInfoService.get(subject);
            user.setPhone(telephone);
            user.setUpdateBy(subject);
            user.setUpdateTime(new Date());
            user.setPhoneLastUpdateTime(new Date());

            int count = personInfoService.update(user);
            if (count > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @PostMapping("createUser")
    @ApiOperation(value = "一键注册(公开接口)")
    public MessageResult<Map> createUser(String openId,
                                           String phone) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            if (StrUtil.isBlank(openId)) {
                throw new Exception("微信标识不存在");
            }

            if (StrUtil.isBlank(phone)) {
                throw new Exception("手机号不能为空");
            }

            PersonInfo personInfo = null;
            String token = null;
            if (StringUtils.isNotEmpty(openId)) {
                personInfo = personInfoService.findByPhone(phone);

                if (personInfo != null) {
                    personInfo.setOpenId(openId);
                    personInfo.setUpdateTime(new Date());
                    personInfoService.update(personInfo);
                    token = JwtUtil.createToken(jwtSecret, personInfo.getId(), DateTime.now().plusHours(6).toDate());
                } else {
                    personInfo = new PersonInfo();
                    personInfo.setId(UUID.randomUUID().toString());
                    personInfo.setDelFlag(false);
                    personInfo.setCreateTime(new Date());
                    personInfo.setPhone(phone);
                    personInfo.setUserName(phone);
                    personInfo.setOpenId(openId);
                    DES3 des3 = new DES3();
                    personInfo.setPassword(des3.encrypt(jwtSecret,"123456"));
                    personInfo.setStatus("0");
                    personInfo.setIsAnonymous(true);
                    personInfo.setIntention("1");//求职中
                    personInfo.setJobStatus("1");//显示简历

                    personInfoService.insert(personInfo);

                    token = JwtUtil.createToken(jwtSecret, personInfo.getId(), DateTime.now().plusHours(6).toDate());
                }
            }

            Map<String, Object> map = new HashMap<>();
            map.put("token", token);
            map.put("regUser", personInfo);

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            ex.printStackTrace();
            log.error(ex.getMessage());
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @ApiOperation(value="绑定认证信息")
    @RequestMapping(value = "createAuthentication",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "realName", value = "真实姓名", required = false, paramType = "form"),
            @ApiImplicitParam(name = "idCard", value = "身份证", required = false, paramType = "form"),
            @ApiImplicitParam(name = "enterpriseName", value = "企业全称", required = false, paramType = "form"),
            @ApiImplicitParam(name = "licenseUrl", value = "营业执照地址", required = false, paramType = "form"),
            @ApiImplicitParam(name = "type", value = "认证类型(0个人,1企业 不传默认0", required = false, paramType = "form"),
            @ApiImplicitParam(name = "address", value = "地址", required = false, paramType = "form"),
            @ApiImplicitParam(name = "contactsPersonId", value = "联系人", required = false, paramType = "form"),
            @ApiImplicitParam(name = "contactsPhone", value = "联系方式", required = false, paramType = "form"),
    })
    public MessageResult<Map> createAuthentication(
            @RequestParam(value="realName",defaultValue="") String realName,
            @RequestParam(value="idCard",defaultValue="") String idCard,
            @RequestParam(value="enterpriseName",defaultValue="") String enterpriseName,
            @RequestParam(value="licenseUrl",defaultValue="") String licenseUrl,
            @RequestParam(value="type",defaultValue="0") String type,
            @RequestParam(value="address",defaultValue="") String address,
            @RequestParam(value="contactsPersonId",defaultValue="") String contactsPersonId,
            @RequestParam(value="contactsPhone",defaultValue="") String contactsPhone,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            if("0".equals(type)) {
                //个人
                personInfo.setStatus("0");
                personInfo.setRealName(realName);
                personInfo.setIdCard(idCard);
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());
                personInfo.setApplicationCheckTime(new Date());

                personInfoService.update(personInfo);
            }else if("1".equals(type)){
                if(StringUtils.isNotEmpty(personInfo.getEnterpriseId())) {
                    EnterpriseInfo enterpriseInfo = enterpriseInfoService.get(personInfo.getEnterpriseId());
                    if(enterpriseInfo == null) {
                        enterpriseInfo.setUpdateBy(subject);
                        enterpriseInfo.setUpdateTime(new Date());
                        enterpriseInfo.setLicenseImage(licenseUrl);
                        enterpriseInfo.setName(enterpriseName);
                        enterpriseInfo.setStatus("0");
                        enterpriseInfo.setAddress(address);
                        enterpriseInfo.setContactsPersonId(contactsPersonId);
                        enterpriseInfo.setContactsPhone(contactsPhone);
                        enterpriseInfo.setCheckTime(new Date());//认证时间

                        enterpriseInfoService.update(enterpriseInfo);
                    }else{
                        enterpriseInfo = new EnterpriseInfo();
                        enterpriseInfo.setId(UUID.randomUUID().toString());
                        enterpriseInfo.setDelFlag(false);
                        enterpriseInfo.setCreateBy(subject);
                        enterpriseInfo.setCreateTime(new Date());
                        enterpriseInfo.setLicenseImage(licenseUrl);
                        enterpriseInfo.setName(enterpriseName);
                        enterpriseInfo.setStatus("0");
                        enterpriseInfo.setAddress(address);
                        enterpriseInfo.setContactsPersonId(contactsPersonId);
                        enterpriseInfo.setContactsPhone(contactsPhone);
                        enterpriseInfo.setCheckTime(new Date());//认证时间

                        int count = enterpriseInfoService.insert(enterpriseInfo);

                        if(count> 0) {
                            personInfo.setEnterpriseId(enterpriseInfo.getId());
                            personInfoService.update(personInfo);
                        }
                    }
                }else{
                    EnterpriseInfo enterpriseInfo = new EnterpriseInfo();
                    enterpriseInfo.setId(UUID.randomUUID().toString());
                    enterpriseInfo.setDelFlag(false);
                    enterpriseInfo.setCreateBy(subject);
                    enterpriseInfo.setCreateTime(new Date());
                    enterpriseInfo.setLicenseImage(licenseUrl);
                    enterpriseInfo.setName(enterpriseName);
                    enterpriseInfo.setStatus("0");
                    enterpriseInfo.setAddress(address);
                    enterpriseInfo.setContactsPersonId(contactsPersonId);
                    enterpriseInfo.setContactsPhone(contactsPhone);
                    enterpriseInfo.setCheckTime(new Date());//认证时间

                    int count = enterpriseInfoService.insert(enterpriseInfo);

                    if(count> 0) {
                        personInfo.setEnterpriseId(enterpriseInfo.getId());
                        personInfoService.update(personInfo);
                    }
                }
            }


            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="修改求职意向(1求职中,2已入职")
    @RequestMapping(value = "updateIntention",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "intention", value = "求职意向(1求职中,2已入职", required = false, paramType = "form"),
    })
    public MessageResult<Map> updateIntention(
            @RequestParam(value="intention",defaultValue="1") String intention,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            personInfo.setIntention(intention);
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());
            personInfoService.update(personInfo);

            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }

    @ApiOperation(value="修改求职意向(1求职中,2已入职")
    @RequestMapping(value = "updatePerson",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "intention", value = "求职意向(1求职中,2已入职", required = false, paramType = "form"),
            @ApiImplicitParam(name = "isAnonymous", value = "是否匿名", required = false, paramType = "form"),
            @ApiImplicitParam(name = "jobStatus", value = "简历公开状态", required = false, paramType = "form"),
    })
    public MessageResult<Map> updatePerson(
            @RequestParam(value="intention",defaultValue="") String intention,
            @RequestParam(value="isAnonymous",defaultValue="") Boolean isAnonymous,
            @RequestParam(value="jobStatus",defaultValue="") String jobStatus,
            @RequestAttribute String subject){

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            PersonInfo personInfo = personInfoService.get(subject);
            if (personInfo == null) {
                throw new Exception("未登录");
            }

            if(StringUtils.isNotEmpty(intention)){
                personInfo.setIntention(intention);
            }

            if(isAnonymous != null){
                personInfo.setIsAnonymous(isAnonymous);
            }

            if(StringUtils.isNotEmpty(jobStatus)){
                personInfo.setJobStatus(jobStatus);
            }

            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());
            personInfoService.update(personInfo);

            msgResult.setResult(true);
        }catch (Exception e){
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
            e.printStackTrace();
        }

        return msgResult;
    }


    @ApiOperation(value="保存登陆记录(免登陆")
    @RequestMapping(value = "saveLoginRecord",method = RequestMethod.POST)
    public MessageResult<Map> saveLoginRecord(@RequestParam(value="openId",defaultValue="") String openId){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            PersonInfo regUser= personInfoService.findByOpenId(openId);
            if (regUser != null) {
                PersonLoginLog personLoginLog = new PersonLoginLog();
                personLoginLog.setId(UUID.randomUUID().toString());
                personLoginLog.setDelFlag(false);
                personLoginLog.setCreateBy(regUser.getId());
                personLoginLog.setCreateTime(new Date());
                personLoginLog.setPersonId(regUser.getId());
                personLoginLogService.insert(personLoginLog);


                //更新简历
                JobInformationInfo jobInformationInfo = jobInformationInfoService.findByPersonId(regUser.getId());
                if(jobInformationInfo != null) {
                    jobInformationInfo.setUpdateTime(new Date());
                    jobInformationInfoService.update(jobInformationInfo);
                }
            }
            msgResult.setResult(true);
        } catch (Exception ex) {
            if (ex instanceof CustomException) {
                log.error(ex.getMessage());
            } else {
                log.error(ex.getMessage(), ex);
            }

            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }
}
