package com.jpsoft.employment.modules.wechat.controller;

import com.alibaba.fastjson.JSONObject;
import com.jpsoft.employment.config.WxProperties;
import com.jpsoft.employment.config.WxPropertiesApplet;
import com.jpsoft.employment.modules.common.dto.MessageResult;
import com.jpsoft.employment.modules.common.utils.*;
import com.jpsoft.employment.modules.wechat.entity.AccessToken;
import com.jpsoft.employment.modules.wechat.entity.SessionKey;
import com.jpsoft.employment.modules.wechat.vo.UserInfo;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@RequestMapping("/wechat")
@RestController
public class WxController {
    @Autowired
    private WxProperties wxProperties;
    @Autowired
    private WxPropertiesApplet wxPropertiesApplet;
    @Autowired
    private ValueOperations<String, Object> valueOperations;

    @ApiOperation(value = "获取微信配置")
    @GetMapping(value = "/getConfig")
    public MessageResult getConfig(String url) {

        int code = 200;
        String message = "获取成功";
        Object data = "";
        boolean result = true;

        try {
            String accessToken = (String)valueOperations.get("accessToken");
            if(StringUtils.isEmpty(accessToken)) {
                AccessToken token = WeixinUtil.getAccessToken(wxProperties.getAppId(), wxProperties.getAppSecret());
                valueOperations.set("accessToken",token.getToken(),2, TimeUnit.HOURS);
                accessToken = token.getToken();
            }

            Map<String, String> wxMap = Sign.sign(WeixinUtil.getJsAPI(accessToken), url);

            wxMap.put("appId", wxProperties.getAppId());

            log.warn(JSONObject.toJSONString(wxMap));

            HashMap<String, Object> dataMap = new HashMap<String, Object>();

            dataMap.put("wxConfig", wxMap);

            data = dataMap;
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);

            code = 500;
            result = false;
            message = "系统错误";
        }

        return new MessageResult(result, message, data, code);
    }

    @ApiOperation(value = "获取微信code")
    @PostMapping(value = "findWechatUrl")
    public MessageResult findWechatUrl(@RequestBody String url) {

        String newpath = WeixinUtil.getCodeRequest(wxProperties.getAppId(), url, "snsapi_userinfo");

        HashMap<String, Object> dataMap = new HashMap<String, Object>();

        dataMap.put("wechatUrl", newpath);

        return new MessageResult(true, "获取成功", dataMap, 200);
    }

    @ApiOperation(value = "获取公众号用户信息")
    @GetMapping(value = "findUserInfo/{code}")
    public MessageResult findUserInfo(@PathVariable String code) {
        try {
            log.warn("code=" + code);
            log.warn("appId=" + wxProperties.getAppId());
            log.warn("appSecret=" + wxProperties.getAppSecret());

            AccessToken at = WeixinUtil.getAccessToken(wxProperties.getAppId(), wxProperties.getAppSecret(), code);


            if (at != null && StringUtils.isNotBlank(at.getOpenid())) {
                String openId = at.getOpenid();
                System.out.println("openId:" + openId);

                UserInfo userInfo = WeixinUtil.getUserInfo(openId, at.getToken());

                HashMap<String, Object> dataMap = new HashMap<String, Object>();

                dataMap.put("userInfo", userInfo);

                return new MessageResult(true, "获取微信信息成功", userInfo, 200);

            } else {
                return new MessageResult(false, "获取微信信息失败", "", 400);
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            return new MessageResult(false, "系统错误", "", 500);
        }
    }

    @ApiOperation(value = "获取公众号用户信息")
    @GetMapping(value = "findAppletUserInfo/{code}")
    public MessageResult findAppletUserInfo(@PathVariable String code) {
        try {
            log.warn("code=" + code);
            log.warn("appId=" + wxPropertiesApplet.getAppId());
            log.warn("appSecret=" + wxPropertiesApplet.getAppSecret());

            SessionKey sessionKey = WeixinUtil.getSessionKey(wxPropertiesApplet.getAppId(), wxPropertiesApplet.getAppSecret(), code);


            if (sessionKey != null && StringUtils.isNotBlank(sessionKey.getOpenid())) {
                String openId = sessionKey.getOpenid();
                System.out.println("openId:" + openId);
                HashMap<String, Object> dataMap = new HashMap<String, Object>();
                dataMap.put("openId", openId);

//                UserInfo userInfo = WeixinUtil.getUserInfo(openId, sessionKey.getSessionKey());
//
//                HashMap<String, Object> dataMap = new HashMap<String, Object>();
//
//                dataMap.put("userInfo", userInfo);

                return new MessageResult(true, "获取信息成功", openId, 200);

            } else {
                return new MessageResult(false, "获取信息失败", "", 400);
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            return new MessageResult(false, "系统错误", "", 500);
        }
    }

    @ApiOperation(value = "判断用户是否关注公众号")
    @PostMapping(value = "checkSubscribe")
    public MessageResult checkSubscribe(String openId) {
        try {
            String accessToken = (String)valueOperations.get("accessToken");
            if(StringUtils.isEmpty(accessToken)){
                AccessToken token = WeixinUtil.getAccessToken(wxProperties.getAppId(), wxProperties.getAppSecret());
                accessToken = token.getToken();
                valueOperations.set("accessToken",token.getToken(),2, TimeUnit.HOURS);
            }

            String subscribe = WeixinUtil.checkSubscribe(openId,accessToken);
            if(StringUtils.isNotEmpty(subscribe)){
                return new MessageResult(true, "获取成功", subscribe, 200);
            }else{
                return new MessageResult(false, "获取失败", subscribe, 500);
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            return new MessageResult(false, "系统错误", "", 500);
        }
    }

    @GetMapping(value = "getAppId/{id}")
    @ResponseBody
    public MessageResult getAppId(@PathVariable("id") String id) {

        int code = 200;
        String message = "获取成功";
        Object data = "";
        boolean result = true;

        try {

            HashMap<String, Object> wxMap = new HashMap<>();

            wxMap.put("appId", wxProperties.getAppId());

            HashMap<String, Object> dataMap = new HashMap<String, Object>();

            dataMap.put("wxConfig", wxMap);

            data = dataMap;

        } catch (Exception ex) {
            code = 500;
            message = "系统错误";
            result = false;
        }

        return new MessageResult(result, message, data, code);
    }

    @ApiOperation(value = "验证服务器地址的有效性")
    @GetMapping("/")
    @ResponseBody
    public String index(@RequestParam(defaultValue = "") String signature,
                        @RequestParam(defaultValue = "") String timestamp,
                        @RequestParam(defaultValue = "") String nonce,
                        @RequestParam(defaultValue = "") String echostr) {
        log.warn("signature=" + signature);
        log.warn("timestamp=" + timestamp);
        log.warn("nonce=" + nonce);
        log.warn("echostr=" + echostr);

        if (WeixinUtil.checkSignature(wxProperties.getToken(), signature, timestamp, nonce)) {
            return echostr;
        } else {
            return "index";
        }
    }

    @ApiOperation(value = "获取小程序用户手机号信息")
    @GetMapping(value = "findUserPhoneNumber/{code}")
    public MessageResult findUserPhoneNumber(@PathVariable String code) {
        try {

            String accessToken = (String)valueOperations.get("accessToken");
            if(StringUtils.isEmpty(accessToken)){
                AccessToken token = WeixinUtil.getAccessToken(wxPropertiesApplet.getAppId(), wxPropertiesApplet.getAppSecret());
                accessToken = token.getToken();
                valueOperations.set("accessToken",token.getToken(),2, TimeUnit.HOURS);
            }
            String phone = WeixinUtil.getUserPhoneNumber(accessToken,code);

            return new MessageResult(true, "成功", phone, 200);
        } catch (Exception ex) {
            ex.printStackTrace();
            return new MessageResult(false, "系统错误", "", 500);
        }
    }

    @ApiOperation(value = "解密")
    @PostMapping(value = "decryptData")
    @ResponseBody
    public MessageResult decryptData(@RequestParam(defaultValue = "") String encryptedData,
                        @RequestParam(defaultValue = "") String iv,
                        @RequestParam(defaultValue = "") String code) {
        try {
//            String accessToken = (String)valueOperations.get("accessToken");
//            if(StringUtils.isEmpty(accessToken)){
//                AccessToken token = WeixinUtil.getAccessToken(wxPropertiesApplet.getAppId(), wxPropertiesApplet.getAppSecret());
//                accessToken = token.getToken();
//                valueOperations.set("accessToken",token.getToken(),2, TimeUnit.HOURS);
//            }

            String phone = "";
            SessionKey sessionKey = WeixinUtil.getSessionKey(wxPropertiesApplet.getAppId(), wxPropertiesApplet.getAppSecret(),code);
            encryptedData = encryptedData.replace(" ", "+");
            String data = AESDATAUtil.decrypt(encryptedData, sessionKey.getSessionKey(), iv);
            if(StringUtils.isNotEmpty(data)) {
                JSONObject userInfo = JSONObject.parseObject(data);
                phone = userInfo.getString("purePhoneNumber");
                return new MessageResult(true, "成功", phone, 200);
            }else {
                return new MessageResult(false, "获取失败", "", 500);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return new MessageResult(false, "系统错误", ex.getMessage(), 500);
        }
    }
}
