package com.jpsoft.employment.scheduled;

import com.jpsoft.employment.modules.base.entity.GenerateReport;
import com.jpsoft.employment.modules.base.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

@Component
@Slf4j
@Profile(value = {"test","dev","production"})
public class ReportTask {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PersonLoginLogService personLoginLogService;

    @Autowired
    private RecruitPersonRelationService recruitPersonRelationService;

    @Autowired
    private EnterpriseInfoService enterpriseInfoService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private RecruitInformationInfoService recruitInformationInfoService;

    @Autowired
    private JobInformationInfoService jobInformationInfoService;

    @Autowired
    private GenerateReportService generateReportService;

    @Scheduled(cron = "0 0 01 * * ?")
    public void run() {
        try {
            Date now = new Date();

            Calendar cal=Calendar.getInstance();
            cal.setTime(now);
            cal.add(Calendar.DATE,-1);
            Date yesterday = cal.getTime();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String date = sdf.format(yesterday);

            Map<String, Object> searchParams = new HashMap<>();

            Map<String, Object> searchParams1 = new HashMap<>();

            if (StringUtils.isNotEmpty(date)) {
                String startTime = date + " 00:00:00";
                String endTime = date + " 23:59:59";
                searchParams.put("startTime", startTime);
                searchParams.put("endTime", endTime);


                searchParams1.put("endTime", endTime);
            }

            GenerateReport currGenerateReport = generateReportService.findByDate(searchParams);

            if(currGenerateReport==null) {

                //登录人数，新增人数
                Integer loginNum = personLoginLogService.dailyReportList(searchParams);

                //登录人数，累计人数
                List<String> personIdList = personLoginLogService.totalDailyReportList(searchParams1);
                Integer totalLoginNum = personIdList.size();

                //今日注册人数
                Integer dailyPersonRegNum = personInfoService.dailyPersonReg(searchParams);

                //累计注册人数
                Integer totalPersonRegNum = personInfoService.dailyPersonReg(searchParams1);

                //认证人数，新增人数
                Integer personNum = personInfoService.dailyReportList(searchParams);

                //认证人数，累计人数
                Integer totalPersonNum = personInfoService.dailyReportList(searchParams1);

                //简历数 累计简历数
                Integer totalJobInformation = jobInformationInfoService.countList(searchParams1);

                //认证企业数，新增数
                Integer enterpriseNum = enterpriseInfoService.dailyReportList(searchParams);

                //认证企业数，累计数
                Integer totalEnterpriseNum = enterpriseInfoService.dailyReportList(searchParams1);

                //累计岗位数量
                Integer totalRecruitStatusNum = recruitInformationInfoService.dailyReportList(searchParams1);

                //累计岗位需求人数
                Integer totalJobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);

                //新增岗位数量
                searchParams.put("status", "1");
                searchParams.put("isOnline", true);
                searchParams.put("delFlag", false);
                Integer recruitNum = recruitInformationInfoService.dailyReportList(searchParams);

                //实时岗位数量
                searchParams1.put("status", "1");
                searchParams1.put("isOnline", true);
                searchParams1.put("delFlag", false);

                //岗位需求人数
                Integer jobRequirementNum = recruitInformationInfoService.sumJobRequirementNum(searchParams1);


                Integer totalRecruitNum = recruitInformationInfoService.dailyReportList(searchParams1);

                //今日报名人数
                searchParams = new HashMap<>();

                if (StringUtils.isNotEmpty(date)) {
                    String startTime = date + " 00:00:00";
                    String endTime = date + " 23:59:59";
                    searchParams.put("startTime", startTime);
                    searchParams.put("endTime", endTime);
                }
                Integer jobInformationNum = recruitPersonRelationService.dailyReportList(searchParams);


                GenerateReport generateReport = new GenerateReport();
                generateReport.setId(UUID.randomUUID().toString());
                generateReport.setCreateTime(new Date());
                generateReport.setDelFlag(false);
                generateReport.setDate(sdf.parse(date));
                generateReport.setLoginNum(loginNum);
                generateReport.setTotalLoginNum(totalLoginNum);
                generateReport.setDailyPersonRegNum(dailyPersonRegNum);
                generateReport.setTotalPersonRegNum(totalPersonRegNum);
                generateReport.setPersonNum(personNum);
                generateReport.setTotalPersonNum(totalPersonNum);
                generateReport.setTotalJobInformation(totalJobInformation);
                generateReport.setEnterpriseNum(enterpriseNum);
                generateReport.setTotalEnterpriseNum(totalEnterpriseNum);
                generateReport.setTotalRecruitStatusNum(totalRecruitStatusNum);
                generateReport.setTotalJobRequirementNum(totalJobRequirementNum);
                generateReport.setRecruitNum(recruitNum);
                generateReport.setJobRequirementNum(jobRequirementNum);
                generateReport.setTotalRecruitNum(totalRecruitNum);
                generateReport.setJobInformationNum(jobInformationNum);

                int count = generateReportService.insert(generateReport);
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
        }
    }
}
